@@ -93,8 +93,8 @@ class AIServices(WMLResource):
                 client._ai_services.ConfigurationMetaNames.NAME: "AI service example",
                 client._ai_services.ConfigurationMetaNames.DESCRIPTION: "This is AI service function",
                 client._ai_services.ConfigurationMetaNames.SOFTWARE_SPEC_ID: "53dc4cf1-252f-424b-b52d-5cdd9814987f",
-                client._ai_services.ConfigurationMetaNames.REQUEST_DOCUMENTATION: request_documentation,
-                client._ai_services.ConfigurationMetaNames.RESPONSE_DOCUMENTATION: response_documentation
+                client._ai_services.ConfigurationMetaNames.REQUEST_DOCUMENTATION: documentation_request,
+                client._ai_services.ConfigurationMetaNames.RESPONSE_DOCUMENTATION: documentation_response,
                 }
 
             def deployable_ai_service(context, params={"k1":"v1"}, **kwargs):
@@ -246,7 +246,7 @@ class AIServices(WMLResource):
         details = self.get_details(ai_service_id)
 
         patch_payload = self.ConfigurationMetaNames._generate_patch_payload(
-            details["entity"], changes, with_validation=True
+            details, changes, with_validation=True
         )
 
         url = self._client.service_instance._href_definitions.get_ai_service_href(
@@ -579,29 +579,14 @@ class AIServices(WMLResource):
             get_all=self._should_get_all_values(limit)
         )["resources"]
 
-        sw_spec_info = {
-            s["id"]: s
-            for s in self._client.software_specifications.get_details(state_info=True)[
-                "resources"
-            ]
-        }
-
-        def get_spec_info(spec_id: str | None, prop: str) -> str:
-            if spec_id and spec_id in sw_spec_info:
-                return sw_spec_info[spec_id].get(prop, "")
-            else:
-                return ""
-
         ai_service_values = [
             (
                 m["metadata"]["id"],
                 m["metadata"]["name"],
                 m["metadata"]["created_at"],
                 m["entity"]["code_type"],
-                get_spec_info(m["entity"].get("software_spec", {}).get("id"), "state"),
-                get_spec_info(
-                    m["entity"].get("software_spec", {}).get("id"), "replacement"
-                ),
+                self._client.software_specifications._get_state(m),
+                self._client.software_specifications._get_replacement(m),
             )
             for m in ai_services_resources
         ]
