@@ -5,6 +5,7 @@
 from __future__ import annotations
 
 import os
+from enum import Enum
 from typing import Any, TYPE_CHECKING
 from warnings import warn
 
@@ -12,6 +13,7 @@ from ibm_watsonx_ai._wrappers import requests
 from ibm_watsonx_ai.messages.messages import Messages
 from ibm_watsonx_ai.metanames import AssetsMetaNames
 from ibm_watsonx_ai.utils import DATA_ASSETS_DETAILS_TYPE
+from ibm_watsonx_ai.utils.enums import AssetDuplicateAction
 from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid
 from ibm_watsonx_ai.wml_client_error import WMLClientError, ApiRequestFailure
 from ibm_watsonx_ai.wml_resource import WMLResource
@@ -70,7 +72,12 @@ class Assets(WMLResource):
             get_all=get_all,
         )
 
-    def create(self, name: str, file_path: str) -> dict[str, Any]:
+    def create(
+        self,
+        name: str,
+        file_path: str,
+        duplicate_action: AssetDuplicateAction | None = None,
+    ) -> dict[str, Any]:
         """Create a data asset and upload content to it.
 
         :param name: name to be given to the data asset
@@ -79,6 +86,10 @@ class Assets(WMLResource):
         :param file_path: path to the content file to be uploaded
         :type file_path: str
 
+        :param duplicate_action: determines behaviour when asset with the same name already exists,
+            if not specified, the value from catalogs/projects/spaces will be used
+        :type duplicate_action: AssetDuplicateAction, optional
+
         :return: metadata of the stored data asset
         :rtype: dict
 
@@ -93,7 +104,15 @@ class Assets(WMLResource):
         # TODO add error handling and cleaning (remove the file)
         Assets._validate_type(name, "name", str, True)
         Assets._validate_type(file_path, "file_path", str, True)
-        return self._create_asset(name, file_path)
+        return self._create_asset(
+            name,
+            file_path,
+            duplicate_action=(
+                duplicate_action.value
+                if isinstance(duplicate_action, Enum)
+                else duplicate_action
+            ),
+        )
 
     def store(self, meta_props: dict) -> dict[str, Any]:
         """Create a data asset and upload content to it.
@@ -166,8 +185,20 @@ class Assets(WMLResource):
         if self.ConfigurationMetaNames.DESCRIPTION in meta_props:
             description = meta_props[self.ConfigurationMetaNames.DESCRIPTION]
 
+        duplicate_action = meta_props.pop(
+            self.ConfigurationMetaNames.DUPLICATE_ACTION, None
+        )
+
         return self._create_asset(
-            name, file_path, connection_id=connection_id, description=description
+            name,
+            file_path,
+            connection_id=connection_id,
+            description=description,
+            duplicate_action=(
+                duplicate_action.value
+                if isinstance(duplicate_action, Enum)
+                else duplicate_action
+            ),
         )
 
     def _create_asset(
@@ -176,6 +207,7 @@ class Assets(WMLResource):
         file_path: str,
         connection_id: str | None = None,
         description: str | None = None,
+        duplicate_action: str | None = None,
     ) -> dict:
         ##Step1: Create a data asset
         desc = description
@@ -204,13 +236,18 @@ class Assets(WMLResource):
         if connection_id is not None:
             asset_meta["metadata"].update({"tags": ["connected-data"]})
 
+        params = self._client._params()
+
+        if duplicate_action:
+            params["duplicate_action"] = duplicate_action
+
         # Step1  : Create an asset
         print(Messages.get_message(message_id="creating_data_asset"))
 
         creation_response = requests.post(
             self._client.service_instance._href_definitions.get_data_assets_href(),
             headers=self._client._get_headers(),
-            params=self._client._params(),
+            params=params,
             json=asset_meta,
         )
 
@@ -566,13 +603,20 @@ class Assets(WMLResource):
         )
 
     def delete(
-        self, asset_id: str | None = None, **kwargs: Any
+        self,
+        asset_id: str | None = None,
+        purge_on_delete: bool | None = None,
+        **kwargs: Any,
     ) -> dict | str:  # asset_id is optional for backward compatibility
-        """Delete a stored data asset.
+        """Soft delete the stored data asset. The asset will be moved to trashed assets
+        and will not be visible in asset list. To permanently delete assets set `purge_on_delete` parameter to True.
 
         :param asset_id: unique ID of the data asset
         :type asset_id: str
 
+        :param purge_on_delete: if set to True will purge the asset
+        :type purge_on_delete: bool, optional
+
         :return: status ("SUCCESS" or "FAILED") or dictionary, if deleted asynchronously
         :rtype: str or dict
 
@@ -587,9 +631,14 @@ class Assets(WMLResource):
 
         Assets._validate_type(asset_id, "asset_id", str, True)
 
+        params = self._client._params()
+
+        if purge_on_delete:
+            params["purge_on_delete"] = True
+
         response = requests.delete(
             self._client.service_instance._href_definitions.get_asset_href(asset_id),
-            params=self._client._params(),
+            params=params,
             headers=self._client._get_headers(),
         )
         if response.status_code == 200:
