@@ -18,10 +18,23 @@ from warnings import warn
 from typing import Any, cast, TypeAlias
 import json
 import base64
+import httpx
 
 import ibm_watsonx_ai.utils
+from ibm_watsonx_ai.projects import Projects
+from ibm_watsonx_ai.trashed_assets import TrashedAssets
+from ibm_watsonx_ai._wrappers.requests import (
+    _get_httpx_client,
+    _httpx_transport_params,
+    _get_async_client,
+)
+from ibm_watsonx_ai.utils.auth import get_auth_method
 from ibm_watsonx_ai.utils import get_user_agent_header
-from ibm_watsonx_ai.utils.utils import _APIClientSession
+from ibm_watsonx_ai.utils.utils import (
+    _APIClientSession,
+    HttpClientConfig,
+    DEFAULT_HTTP_CLIENT_CONFIG,
+)
 from ibm_watsonx_ai.Set import Set
 from ibm_watsonx_ai.assets import Assets
 from ibm_watsonx_ai.connections import Connections
@@ -63,8 +76,9 @@ class APIClient:
     """The main class of ibm_watsonx_ai. The very heart of the module. APIClient contains objects that manage the service reasources.
 
     To explore how to use APIClient, refer to:
-     - :ref:`Setup<setup>` - to check correct initialization of APIClient for a specific environment.
-     - :ref:`Core<core>` - to explore core properties of an APIClient object.
+
+    - :ref:`Setup<setup>` - to check correct initialization of APIClient for a specific environment.
+    - :ref:`Core<core>` - to explore core properties of an APIClient object.
 
     :param url: URL of the service
     :type url: str
@@ -81,6 +95,58 @@ class APIClient:
     :param verify: certificate verification flag, deprecated, use Credentials(verify=...) to set `verify`
     :type verify: bool, optional
 
+    :param httpx_client: A customizable `httpx.Client` for ModelInference, Embeddings and methods related to the deployments management and scoring.
+        The `httpx.Client` is used to improve performance across deployments, foundation models, and embeddings. This parameter accepts two types of input:
+
+        - A direct instance of `httpx.Client()`
+        - A set of parameters provided via the `HttpClientConfig` class
+
+        **Example:**
+
+        .. code-block:: python
+
+            from ibm_watsonx_ai.utils.utils import HttpClientConfig
+
+            limits=httpx.Limits(
+                max_connections=5
+            )
+            timeout = httpx.Timeout(7)
+            http_config = HttpClientConfig(timeout=timeout, limits=limits)
+
+        If not provided, a default instance of `httpx.Client` is created.
+
+        .. note::
+            If you need to adjust timeouts or limits, using ``HttpClientConfig`` is the recommended approach.
+            When the ``proxies`` parameter is provided in credentials, ``httpx.Client`` will use these proxies.
+            However, if you want to create a separate ``httpx.Client``, all parameters must be provided by the user.
+    :type httpx_client: httpx.Client, HttpClientConfig, optional
+
+    :param async_httpx_client: A customizable `httpx.AsyncClient` for ModelInference. The `httpx.AsyncClient` is used to improve performance of foundation models inference. This parameter accepts two types of input:
+
+        - A direct instance of `httpx.AsyncClient`
+        - A set of parameters provided via the `HttpClientConfig` class
+
+        **Example:**
+
+        .. code-block:: python
+
+            from ibm_watsonx_ai.utils.utils import HttpClientConfig
+
+            limits=httpx.Limits(
+                max_connections=5
+            )
+            timeout = httpx.Timeout(7)
+            http_config = HttpClientConfig(timeout=timeout, limits=limits)
+
+        If not provided, a default instance of `httpx.AsyncClient` is created.
+
+        .. note::
+            If you need to adjust timeouts or limits, using ``HttpClientConfig`` is the recommended approach.
+            When the ``proxies`` parameter is provided in credentials, ``httpx.Client`` will use these proxies.
+            However, if you want to create a separate ``httpx.Client``, all parameters must be provided by the user.
+
+    :type async_httpx_client: httpx.AsyncClient, HttpClientConfig, optional
+
     **Example:**
 
     .. code-block:: python
@@ -112,6 +178,10 @@ class APIClient:
         project_id: str | None = None,
         space_id: str | None = None,
         verify: str | bool | None = None,
+        httpx_client: httpx.Client | HttpClientConfig = DEFAULT_HTTP_CLIENT_CONFIG,
+        async_httpx_client: (
+            httpx.AsyncClient | HttpClientConfig
+        ) = DEFAULT_HTTP_CLIENT_CONFIG,
         **kwargs: Any,
     ) -> None:
         if (wml_credentials := kwargs.get("wml_credentials")) is not None:
@@ -391,6 +461,9 @@ class APIClient:
                 self.CPD_version >= 5.0
             )
 
+            self._auth_method = get_auth_method(self)
+            self._auth_method.get_token()
+
             # For cloud, service_instance.details will be set during space creation( if instance is associated ) or
             # while patching a space with an instance
 
@@ -410,12 +483,14 @@ class APIClient:
                 self.set.default_space(space_id)
 
             self.spaces = Spaces(self)
+            self.projects = Projects(self)
 
             self.export_assets = Export(self)
             self.import_assets = Import(self)
 
             if self.ICP_PLATFORM_SPACES:
                 self.shiny = Shiny(self)
+                self.trashed_assets = TrashedAssets(self)
 
             self.script = Script(self)
             self.model_definitions = ModelDefinition(self)
@@ -455,6 +530,26 @@ class APIClient:
                 Messages.get_message(message_id="client_successfully_initialized")
             )
 
+            if isinstance(httpx_client, HttpClientConfig):
+                httpx_client = _get_httpx_client(
+                    transport_params=_httpx_transport_params(
+                        self, limits=httpx_client.limits
+                    ),
+                    timeout=httpx_client.timeout,
+                )
+
+            self._httpx_client = httpx_client
+
+            if isinstance(async_httpx_client, HttpClientConfig):
+                async_httpx_client = _get_async_client(
+                    transport_params=_httpx_transport_params(
+                        self, limits=async_httpx_client.limits
+                    ),
+                    timeout=async_httpx_client.timeout,
+                )
+
+            self._async_httpx_client = async_httpx_client
+
     @property
     def wml_credentials(self) -> dict[str, str]:
         wml_credentials_attribute_deprecated = (
@@ -491,11 +586,11 @@ class APIClient:
 
     @property
     def token(self) -> str:
-        return self.service_instance._auth_method.get_token()
+        return self._auth_method.get_token()
 
     @token.setter
     def token(self, value: str) -> None:
-        self.service_instance._auth_method._token = value
+        self._auth_method._token = value
 
     @property
     def _session(self) -> RequestsLikeType:
@@ -529,7 +624,27 @@ class APIClient:
             ),
             category=DeprecationWarning,
         )
-        return isinstance(self.service_instance._auth_method, TokenAuth)
+        return isinstance(self._auth_method, TokenAuth)
+
+    @property
+    def httpx_client(self) -> httpx.Client:
+        return self._httpx_client
+
+    @httpx_client.setter
+    def httpx_client(self, value: httpx.Client):
+        if self._httpx_client:
+            self._httpx_client.close()
+        self._httpx_client = value
+
+    @property
+    def async_httpx_client(self) -> httpx.AsyncClient:
+        return self._async_httpx_client
+
+    @async_httpx_client.setter
+    def async_httpx_client(self, value: httpx.AsyncClient):
+        if self._async_httpx_client:
+            self._async_httpx_client.aclose()
+        self._async_httpx_client = value
 
     @staticmethod
     def _get_api_version_param() -> str:
@@ -622,10 +737,6 @@ class APIClient:
         if not zen:
             headers["User-Agent"] = get_user_agent_header()
 
-            if not self.ICP_PLATFORM_SPACES:
-                if self.default_project_id is not None:
-                    headers["X-Watson-Project-ID"] = self.default_project_id
-
         if not self.generate_ux_tag:
             headers.update({"X-WX-UX": "true"})
             self.generate_ux_tag = True
@@ -642,6 +753,29 @@ class APIClient:
 
         return headers
 
+    def get_headers(
+        self,
+        content_type: str | None = "application/json",
+        include_user_agent: bool = False,
+    ) -> dict:
+        """Get HTTP headers used during requests.
+
+        :param content_type: value for `Content-Type` header, defaults to `application/json`
+        :type name: str, optional
+
+        :param include_user_agent: whether the result should include `User-Agent` header, defaults to `False`
+        :type name: bool, optional
+
+        :return: headers used during requests
+        :rtype: dict
+        """
+
+        return self._get_headers(
+            content_type=content_type or "",
+            no_content_type=content_type is None,
+            zen=not include_user_agent,
+        )
+
     def set_token(self, token: str) -> None:
         """
         Method which allows refresh/set new User Authorization Token.
@@ -663,7 +797,21 @@ class APIClient:
         self.credentials.token = token
         from ibm_watsonx_ai.utils.auth import TokenAuth
 
-        self.service_instance._auth_method = TokenAuth(token)
+        if isinstance(self._auth_method, TokenAuth):
+            self._auth_method.set_token(token)
+        else:
+            # the auth method type was changed to TokenAuth
+            authentication_method_changed_warning = (
+                "Authentication method changed to TokenAuth. "
+                "The token will not be automatically refreshed from this point of time. "
+                "Use `APIClient.set_token` function to manually update token."
+            )
+
+            warn(authentication_method_changed_warning)
+
+            self._auth_method = TokenAuth(
+                token, on_token_set=self.repository._refresh_repo_client
+            )
 
     def set_headers(self, headers: dict) -> None:
         """
@@ -679,7 +827,6 @@ class APIClient:
             headers = {
                 'Authorization': 'Bearer <USER AUTHORIZATION TOKEN>',
                 'User-Agent': 'ibm-watsonx-ai/1.0.1 (lang=python; arch=x86_64; os=darwin; python.version=3.10.13)',
-                'X-Watson-Project-ID': '<PROJECT ID>',
                 'Content-Type': 'application/json'
             }
 
