@@ -60,6 +60,9 @@ class DocumentsIterableDataset(IterableDataset):
         - **pdf** (".pdf" file extension) - standard pdf document
         - **html** (".html" file extension) - saved html side
         - **markdown** (".md" file extension) - plain text formatted with markdown
+        - **pptx** (".pptx" file extension) - standard PowerPoint style file
+        - **json** (".json" file extension) - standard json file
+        - **yaml** (".yaml" file extension) - standard yaml file
 
     :param connections: list of connections to the documents
     :type connections: list[DataConnection]
