@@ -21,7 +21,6 @@ import json
 from warnings import warn
 from enum import Enum
 
-import ibm_watsonx_ai._wrappers.requests as requests
 from ibm_watsonx_ai.utils import (
     print_text_header_h1,
     print_text_header_h2,
@@ -258,7 +257,7 @@ class Deployments(WMLResource):
 
         url = self._client.service_instance._href_definitions.get_deployments_href()
 
-        response = requests.post(
+        response = self._client.httpx_client.post(
             url,
             json=metaProps,
             params=self._client._params(),  # version is mandatory
@@ -340,8 +339,8 @@ class Deployments(WMLResource):
         else:
             error_msg = "Deployment creation failed"
             reason = response.text
-            print(reason)
             print_text_header_h2(error_msg)
+            print(reason)
             raise WMLClientError(
                 error_msg + ". Error: " + str(response.status_code) + ". " + reason
             )
@@ -465,7 +464,9 @@ class Deployments(WMLResource):
         }
 
         url = self._client.service_instance._href_definitions.get_deployments_href()
-        res = requests.get(url, headers=self._client._get_headers(), params=params)
+        res = self._client.httpx_client.get(
+            url, headers=self._client._get_headers(), params=params
+        )
 
         if res.status_code == 409:
             response = res.json()
@@ -572,10 +573,10 @@ class Deployments(WMLResource):
             )
 
             deployment_details = self._get_artifact_details(
-                url,
-                deployment_id,
-                limit,
-                "deployments",
+                base_url=url,
+                id=deployment_id,
+                limit=limit,
+                resource_name="deployments",
                 query_params=query_params,
                 _async=asynchronous,
                 _all=get_all,
@@ -583,7 +584,11 @@ class Deployments(WMLResource):
             )
         else:
             deployment_details = self._get_artifact_details(
-                url, deployment_id, limit, "deployments", query_params=query_params
+                url,
+                deployment_id,
+                limit,
+                "deployments",
+                query_params=query_params,
             )
 
         if (
@@ -719,7 +724,7 @@ class Deployments(WMLResource):
             )
         )
 
-        response_delete = requests.delete(
+        response_delete = self._client.httpx_client.delete(
             deployment_url,
             params=self._client._params(),
             headers=self._client._get_headers(),
@@ -839,7 +844,7 @@ class Deployments(WMLResource):
 
         params = self._client._params()
         del params["space_id"]
-        response_scoring = requests.post(
+        response_scoring = self._client.httpx_client.post(
             scoring_url,
             json=payload,
             params=params,  # version parameter is mandatory
@@ -930,13 +935,6 @@ class Deployments(WMLResource):
         values = []
         index = 0
 
-        sw_spec_info = {
-            s["id"]: s
-            for s in self._client.software_specifications.get_details(state_info=True)[
-                "resources"
-            ]
-        }
-
         def enrich_asset_with_type(asset_details: dict, asset_type: str) -> dict:
             if asset_type:
                 asset_details["metadata"]["asset_type"] = asset_type
@@ -952,12 +950,6 @@ class Deployments(WMLResource):
             for el in resources["resources"]
         }
 
-        def get_spec_info(spec_id: str, prop: str) -> str:
-            if spec_id and spec_id in sw_spec_info:
-                return sw_spec_info[spec_id].get(prop, "")
-            else:
-                return ""
-
         for m in resources:
             # Deployment service currently doesn't support limit querying
             # As a workaround, its filtered in python client
@@ -965,14 +957,9 @@ class Deployments(WMLResource):
             if limit is not None and index == limit:
                 break
 
-            spec_id = (
-                asset_info.get(
-                    m["entity"].get("asset", m["entity"].get("prompt_template"))["id"],
-                    {},
-                )
-                .get("entity", {})
-                .get("software_spec", {})
-                .get("id")
+            asset_details = asset_info.get(
+                m["entity"].get("asset", m["entity"].get("prompt_template"))["id"],
+                {},
             )
 
             if (
@@ -992,8 +979,10 @@ class Deployments(WMLResource):
                         m["entity"]["status"]["state"],
                         m["metadata"]["created_at"],
                         m["entity"].get("deployed_asset_type", "unknown"),
-                        get_spec_info(spec_id, "state"),
-                        get_spec_info(spec_id, "replacement"),
+                        self._client.software_specifications._get_state(asset_details),
+                        self._client.software_specifications._get_replacement(
+                            asset_details
+                        ),
                     )
                 )
 
@@ -1078,6 +1067,7 @@ class Deployments(WMLResource):
         self,
         deployment_id: str | None = None,
         changes: dict | None = None,
+        background_mode: bool = False,
         **kwargs: Any,
     ) -> dict | None:
         """Updates existing deployment metadata. If ASSET is patched, then 'id' field is mandatory
@@ -1092,6 +1082,9 @@ class Deployments(WMLResource):
         :return: metadata of the updated deployment
         :rtype: dict or None
 
+        :param background_mode: indicator whether the update() method will run in the background (async) or not (sync), defaults to False
+        :type background_mode: bool, optional
+
         **Examples**
 
         .. code-block:: python
@@ -1116,10 +1109,6 @@ class Deployments(WMLResource):
         if not self._client.CLOUD_PLATFORM_SPACES and self._client.CPD_version < 4.8:
             self._client._check_if_space_is_set()
 
-        ret202 = False
-
-        ## In case of passing 'AUTO_ONLINE_DEPLOYMENT' as true, we need to poll for deployment to be either 'deploy_success' or 'update_success'.
-
         Deployments._validate_type(deployment_id, "deployment_id", str, True)
 
         if ("asset" in changes and not changes["asset"]) and (
@@ -1159,7 +1148,7 @@ class Deployments(WMLResource):
             print(msg)
             raise WMLClientError(msg)
 
-        deployment_details = self.get_details(deployment_id)
+        deployment_details = self.get_details(deployment_id, _silent=True)
         serving_name_change = False
         new_serving_name = None
         if self.ConfigurationMetaNames.SERVING_NAME in changes:
@@ -1182,19 +1171,11 @@ class Deployments(WMLResource):
                 }
             )
 
-        ## As auto_online_deployment and auto_redeploy values are passed as 'bool' but service needs them in 'str' format to patch.
-        for ele in patch_payload:
-            if (
-                "auto_online_deployment" in ele["path"]
-                or "auto_redeploy" in ele["path"]
-            ):
-                ele["value"] = str(ele["value"]).lower()
-
         url = self._client.service_instance._href_definitions.get_deployment_href(
             deployment_id
         )
 
-        response = requests.patch(
+        response = self._client.httpx_client.patch(
             url,
             json=patch_payload,
             params=self._client._params(),
@@ -1202,79 +1183,77 @@ class Deployments(WMLResource):
         )
 
         if patch_job and response.status_code == 202:
-            updated_details = self._handle_response(
+            deployment_details = self._handle_response(
                 202, "deployment asset patch", response
             )
-
-            ret202 = True
-
             print(
-                f"Since {patch_job_field} is patched, deployment with new asset id/rev is being started. "
-                "Monitor the status using deployments.get_details(deployment_id) api"
+                f"Since {patch_job_field} is patched, deployment need to be restarted. "
             )
+            if background_mode:
+                print(
+                    "Monitor the status using deployments.get_details(deployment_id) api"
+                )
         elif response.status_code == 202:
-            updated_details = self._handle_response(202, "deployment scaling", response)
-            ret202 = True
+            deployment_details = self._handle_response(
+                202, "deployment scaling", response
+            )
         else:
-            updated_details = self._handle_response(200, "deployment patch", response)
+            deployment_details = self._handle_response(
+                200, "deployment patch", response
+            )
 
-        if "auto_online_deployment" in changes:
-            if response is not None:
-                if response.status_code == 200:
-                    deployment_details = self.get_details(deployment_id)
+        if background_mode:
+            return deployment_details
 
-                    import time
+        if response.status_code in (200, 202):
+            deployment_details = self.get_details(deployment_id, _silent=True)
 
-                    print_text_header_h1(
-                        " deployment update for id: '{}' started".format(deployment_id)
-                    )
+            import time
 
+            print_text_header_h1(
+                "Deployment update for id: '{}' started".format(deployment_id)
+            )
+
+            status = deployment_details["entity"]["status"]["state"]
+
+            with StatusLogger(status) as status_logger:
+                while True:
+                    time.sleep(5)
+                    deployment_details = self.get_details(deployment_id, _silent=True)
                     status = deployment_details["entity"]["status"]["state"]
+                    status_logger.log_state(status)
 
-                    with StatusLogger(status) as status_logger:
-                        while True:
-                            time.sleep(5)
-                            deployment_details = self.get_details(deployment_id)
-                            status = deployment_details["entity"]["status"]["state"]
-                            status_logger.log_state(status)
+                    if status != "initializing" and status != "updating":
+                        break
 
-                            if (
-                                status != "DEPLOY_IN_PROGRESS"
-                                and status != "UPDATE_IN_PROGRESS"
-                            ):
-                                break
-
-                    if status == "DEPLOY_SUCCESS" or status == "UPDATE_SUCCESS":
-                        print("")
-                        print_text_header_h2(
-                            "Successfully finished deployment update, deployment_id='{}'".format(
-                                deployment_id
-                            )
-                        )
-                        return deployment_details
-                    else:
-                        print_text_header_h2("Deployment update failed")
-                        if deployment_id is not None:
-                            self._deployment_status_errors_handling(
-                                deployment_details, "update", deployment_id
-                            )
-                else:
-                    error_msg = "Deployment update failed"
-                    reason = response.text
-                    print(reason)
-                    print_text_header_h2(error_msg)
-                    raise WMLClientError(
-                        error_msg
-                        + ". Error: "
-                        + str(response.status_code)
-                        + ". "
-                        + reason
+            if (
+                status == "ready"
+                and "failure" not in deployment_details["entity"]["status"]
+            ):
+                # from apidocs: If any failures, deployment will be reverted back to the previous id/rev and the failure message will be captured in 'failure' field in the response.
+                print("")
+                print_text_header_h2(
+                    "Successfully finished deployment update, deployment_id='{}'".format(
+                        deployment_id
                     )
+                )
+                return deployment_details
+            else:
+                print_text_header_h2("Deployment update failed")
+                if deployment_id is not None:
+                    self._deployment_status_errors_handling(
+                        deployment_details, "update", deployment_id
+                    )
+        else:
+            error_msg = "Deployment update failed"
+            reason = response.text
+            print(reason)
+            print_text_header_h2(error_msg)
+            raise WMLClientError(
+                error_msg + ". Error: " + str(response.status_code) + ". " + reason
+            )
 
-        if not ret202:
-            return updated_details
-
-        return None
+        return deployment_details
 
     ## Below functions are for async scoring. They are just dummy functions.
     def _score_async(
@@ -1304,7 +1283,7 @@ class Deployments(WMLResource):
                 )
             params.update({"retention": retention})
 
-        response_scoring = requests.post(
+        response_scoring = self._client.httpx_client.post(
             scoring_url, params=params, json=scoring_payload, headers=headers
         )
 
@@ -1561,10 +1540,10 @@ class Deployments(WMLResource):
             params["include"] = include
 
         return self._get_artifact_details(
-            url,
-            job_id,
-            limit,
-            "async deployment job" if job_id else "async deployment jobs",
+            base_url=url,
+            id=job_id,
+            limit=limit,
+            resource_name="async deployment job" if job_id else "async deployment jobs",
             query_params=params,
         )
 
@@ -1691,16 +1670,40 @@ class Deployments(WMLResource):
         if job_id is not None and not is_id(job_id):
             raise WMLClientError("'job_id' is not an id: '{}'".format(job_id))
 
+        params = self._client._params()
+
+        if not self._client.CLOUD_PLATFORM_SPACES and self._client.CPD_version <= 5.1:
+            # for CPD 5.1 and lower there is need to use the jobs api directly.
+            # From CPD 5.2.x + and Cloud deployment service will cover the call in DELETE /ml/v4/deployment_jobs
+            # issue: #48242
+            try:
+                job_details = self.get_job_details(job_id=job_id)
+                run_id = job_details["entity"]["platform_job"]["run_id"]
+
+                jobs_runs_url = (
+                    self._client.service_instance._href_definitions.get_jobs_runs_href(
+                        job_id=job_id, run_id=run_id
+                    )
+                )
+
+                response_delete = self._client.httpx_client.delete(
+                    jobs_runs_url, headers=self._client._get_headers(), params=params
+                )
+
+                return self._handle_response(
+                    204, "deployment async job deletion", response_delete, False
+                )
+            except:
+                pass
+
         url = self._client.service_instance._href_definitions.get_async_deployment_jobs_href(
             job_id
         )
 
-        params = self._client._params()
-
         if hard_delete is True:
             params.update({"hard_delete": "true"})
 
-        response_delete = requests.delete(
+        response_delete = self._client.httpx_client.delete(
             url, headers=self._client._get_headers(), params=params
         )
 
@@ -1785,16 +1788,17 @@ class Deployments(WMLResource):
             )
         if inference_type in ["text", "text_stream"]:
             if generated_url not in inference_url_list:
-                for inference_url in inference_url_list:  # Remove on CPD 5.0 release
-                    if (
-                        "v1-beta/deployments" not in inference_url
-                    ):  # Remove on CPD 5.0 release
-                        raise WMLClientError(
-                            Messages.get_message(
-                                deployment_id,
-                                message_id="fm_deployment_has_not_inference_for_generation",
-                            )
+
+                if all(
+                    "/text/generation" not in inference_url
+                    for inference_url in inference_url_list
+                ):
+                    raise WMLClientError(
+                        Messages.get_message(
+                            deployment_id,
+                            message_id="fm_deployment_has_not_inference_for_generation",
                         )
+                    )
 
         return ModelInference(
             deployment_id=deployment_id, params=params, api_client=self._client
@@ -2042,11 +2046,12 @@ class Deployments(WMLResource):
             )
             + "/ai_service"
         )
-
-        response_scoring = requests.post(
+        response_scoring = self._client.httpx_client.post(
             url=scoring_url,
             json=ai_service_payload,
-            params=self._client._params(),  # version parameter is mandatory
+            params=self._client._params(
+                skip_for_create=True, skip_userfs=True
+            ),  # version parameter is mandatory
             headers=self._client._get_headers(),
         )
 
@@ -2088,19 +2093,20 @@ class Deployments(WMLResource):
             + "/ai_service_stream"
         )
 
-        with requests.Session().post(
+        with self._client.httpx_client.stream(
             url=scoring_url,
             json=ai_service_payload,
             headers=self._client._get_headers(),
             params=self._client._params(skip_for_create=True, skip_userfs=True),
-            stream=True,
+            method="POST",
         ) as resp:
             if resp.status_code == 200:
-                for chunk in resp.iter_lines(decode_unicode=True):
+                for chunk in resp.iter_lines():
                     field_name, _, response = chunk.partition(":")
                     if field_name == "data":
                         yield response
             else:
+                resp.read()
                 raise ApiRequestFailure(f"Failure during AI Service run steam", resp)
 
 
@@ -2182,7 +2188,7 @@ class RuntimeContext:
         return self._api_client._get_icptoken()
 
     def get_headers(self) -> dict:
-        """Return headers with refreshed token"""
+        """Return headers with refreshed token."""
         return self._api_client._get_headers()
 
     def get_json(self) -> dict | None:
@@ -2191,3 +2197,7 @@ class RuntimeContext:
         or later as request_payload_json property.
         """
         return self.request_payload_json
+
+    def get_space_id(self) -> str:
+        """Return default space id."""
+        return self._api_client.default_space_id
