@@ -10,7 +10,8 @@ from typing import TypeAlias, TYPE_CHECKING, Any
 from concurrent.futures import ThreadPoolExecutor
 from functools import reduce, partial
 from enum import Enum
-
+from warnings import warn
+import requests as _requests
 import httpx
 
 from ibm_watsonx_ai.wml_client_error import (
@@ -88,6 +89,16 @@ class Embeddings(BaseEmbeddings, WMLResource):
     :param concurrency_limit: number of requests to be sent in parallel when generating embedding vectors, max is 10, defaults to 5
     :type concurrency_limit: int, optional
 
+    :param max_retries: number of retries performed when request was not successful and status code is in retry_status_codes, defaults to 10
+    :type max_retries: int, optional
+
+    :param delay_time: delay time to retry request, factor in exponential backoff formula: wx_delay_time * pow(2.0, attempt), defaults to 0.5s
+    :type delay_time: float, optional
+
+    :param retry_status_codes: list of status codes which will be considered for retry mechanism, defaults to [429, 503, 504, 520]
+    :type retry_status_codes: list[int], optional
+
+
     .. note::
         When the ``credentials`` parameter is passed, one of these parameters is required: [``project_id``, ``space_id``].
 
@@ -134,6 +145,9 @@ class Embeddings(BaseEmbeddings, WMLResource):
         persistent_connection: bool = True,
         batch_size: int = MAX_INPUTS_LENGTH,
         concurrency_limit: int = DEFAULT_CONCURRENCY_LIMIT,
+        max_retries: int | None = None,
+        delay_time: float | None = None,
+        retry_status_codes: list[int] | None = None,
     ) -> None:
         if isinstance(model_id, Enum):
             self.model_id = model_id.value
@@ -188,17 +202,43 @@ class Embeddings(BaseEmbeddings, WMLResource):
 
         WMLResource.__init__(self, __name__, self._client)
 
-        self._transport_params = requests._httpx_transport_params(
-            self._client, retry_status_codes=_RETRY_STATUS_CODES
-        )
+        self._transport_params = requests._httpx_transport_params(self._client)
 
         if self._persistent_connection:
-            self._http_client = _get_httpx_client(
-                transport_params=self._transport_params,
-                timeout=EMBEDDINGS_HTTPX_TIMEOUT,
-            )
+            self._http_client = self._client.httpx_client
         else:
-            self._http_client = requests
+            self._http_client = requests  # type: ignore[assignment]
+            persistent_connection_warn = (
+                "`persistent_connection` is deprecated and will be removed in future. "
+            )
+            warn(persistent_connection_warn, category=DeprecationWarning)
+
+        # Set initially 8 requests per second as it is default for prod instances
+        # if header "x-requests-limit-rate" is different capacity will be updated
+        self.rate_limiter = requests.TokenBucket(rate=8, capacity=8)
+        self.retry_status_codes = retry_status_codes
+        self.max_retries = max_retries
+        self.delay_time = delay_time
+
+    def _generate_raw_response(
+        self,
+        generate_url: str,
+        inputs: list[str],
+        params: ParamsType | None = None,
+        _http_client: httpx.Client | None = None,
+    ) -> httpx.Response | _requests.Response:
+        """Send request with post and return service response."""
+
+        payload = self._prepare_payload(inputs, params)
+
+        post_params: dict[str, Any] = dict(
+            url=generate_url,
+            json=payload,
+            params=self._client._params(skip_for_create=True, skip_userfs=True),
+            headers=self._client._get_headers(),
+        )
+
+        return self._post(_http_client, **post_params)
 
     def generate(
         self,
@@ -232,61 +272,77 @@ class Embeddings(BaseEmbeddings, WMLResource):
         # '(concurrency_limit is DEFAULT_CONCURRENCY_LIMIT) == True' => NO SET
         if concurrency_limit is DEFAULT_CONCURRENCY_LIMIT:
             concurrency_limit = self.concurrency_limit
-        try:
-            # For batch of prompts use keep-alive connection even if persistent_connection=False
-            http_client = (
-                _get_httpx_client(
-                    transport_params=self._transport_params,
+        # For batch of prompts use keep-alive connection even if persistent_connection=False
+        http_client = (
+            self._client.httpx_client
+            if not self._persistent_connection
+            else self._http_client
+        )
+
+        if len(inputs) > self.batch_size:
+            inputs_split = [
+                inputs[i : i + self.batch_size]
+                for i in range(0, len(inputs), self.batch_size)
+            ]
+
+            def make_request(_inputs: list[str]) -> dict:
+                self.rate_limiter.acquire()
+                response = self._generate_raw_response(
+                    generate_url=generate_url,
+                    inputs=_inputs,
+                    params=params,
+                    _http_client=http_client,
                 )
-                if not self._persistent_connection
-                else self._http_client
-            )
-            if len(inputs) > self.batch_size:
-                generated_responses: list = []
-                inputs_splitted = [
-                    inputs[i : i + self.batch_size]
-                    for i in range(0, len(inputs), self.batch_size)
-                ]
-                _generate_partial = partial(
+                rate_limit = int(response.headers.get("x-requests-limit-rate", 0))
+                if rate_limit and rate_limit != self.rate_limiter.capacity:
+                    self.rate_limiter.capacity = rate_limit
+
+                rate_limit_remaining = int(
+                    response.headers.get(
+                        "x-requests-limit-remaining", self.rate_limiter.capacity
+                    )
+                )
+                self.rate_limiter.adjust_tokens(rate_limit_remaining)
+                return self._handle_response(
+                    200,
+                    "generate",
+                    response,
+                    _field_to_hide="embedding",
+                )
+
+            fn = (
+                make_request
+                if self._client.CLOUD_PLATFORM_SPACES
+                else partial(
                     self._generate,
                     generate_url,
                     params=params,
                     _http_client=http_client,
                 )
-                if (inputs_length := len(inputs_splitted)) <= concurrency_limit:
-                    with ThreadPoolExecutor(max_workers=inputs_length) as executor:
-                        generated_responses = list(
-                            executor.map(_generate_partial, inputs_splitted)
-                        )
-                else:
-                    with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
-                        generated_responses = list(
-                            executor.map(_generate_partial, inputs_splitted)
-                        )
-
-                def reduce_response(left: dict, right: dict) -> dict:
-                    import copy
-
-                    left_copy = copy.deepcopy(left)
-                    left_copy["results"].extend(right["results"])
-                    left_copy["input_token_count"] += right["input_token_count"]
-                    return left_copy
-
-                return reduce(
-                    reduce_response, generated_responses[1:], generated_responses[0]
-                )
+            )  # If CDP, don't use Token Bucket
 
+            if (inputs_length := len(inputs_split)) <= concurrency_limit:
+                with ThreadPoolExecutor(max_workers=inputs_length) as executor:
+                    generated_responses = list(executor.map(fn, inputs_split))
             else:
-                results = self._generate(generate_url, inputs, params)
-            return results
-        except Exception:
-            http_client = None  # type: ignore[assignment]
-            raise
-        finally:
-            if not self._persistent_connection and isinstance(
-                http_client, httpx.Client
-            ):
-                http_client.close()
+                with ThreadPoolExecutor(max_workers=concurrency_limit) as executor:
+                    generated_responses = list(executor.map(fn, inputs_split))
+
+            def reduce_response(left: dict, right: dict) -> dict:
+                import copy
+
+                left_copy = copy.deepcopy(left)
+                left_copy["results"].extend(right["results"])
+                left_copy["input_token_count"] += right["input_token_count"]
+                return left_copy
+
+            return reduce(
+                reduce_response, generated_responses[1:], generated_responses[0]
+            )
+
+        else:
+            results = self._generate(generate_url, inputs, params)
+        return results
 
     def embed_documents(
         self,
@@ -372,23 +428,18 @@ class Embeddings(BaseEmbeddings, WMLResource):
         generate_url: str,
         inputs: list[str],
         params: ParamsType | None = None,
-        _http_client: requests.HTTPXClient | None = None,
+        _http_client: requests.HTTPXClient | httpx.Client | None = None,
     ) -> dict:
         """Send request with post and return service response."""
-
-        payload = self._prepare_payload(inputs, params)
-
         http_client = _http_client or self._http_client
-        post_params: dict[str, Any] = dict(
-            url=generate_url,
-            json=payload,
-            params=self._client._params(skip_for_create=True, skip_userfs=True),
-            headers=self._client._get_headers(),
-        )
-        if not isinstance(http_client, httpx.Client):
-            post_params["_retry_status_codes"] = _RETRY_STATUS_CODES
 
-        response_scoring = http_client.post(**post_params)
+        response_scoring = self._generate_raw_response(
+            generate_url=generate_url,
+            inputs=inputs,
+            params=params,
+            _http_client=http_client,
+        )
+
         return self._handle_response(
             200,
             "generate",
@@ -417,12 +468,24 @@ class Embeddings(BaseEmbeddings, WMLResource):
         return data
 
     def close_persistent_connection(self) -> None:
-        """Only applicable if persistent_connection was set to True in Embeddings initialization."""
+        """
+        Only applicable if persistent_connection was set to True in Embeddings initialization.
+        Calling this method closes the current `httpx.Client` and recreates a new `httpx.Client` with default values:
+        timeout: httpx.Timeout(read=30 * 60, write=30 * 60, connect=10, pool=30 * 60)
+        limit: httpx.Limits(max_connections=10, max_keepalive_connections=10, keepalive_expiry=HTTPX_KEEPALIVE_EXPIRY)
+        """
         if self._persistent_connection is not None and isinstance(
             self._http_client, httpx.Client
         ):
             self._http_client.close()
-            self._http_client = _get_httpx_client(
+            self._client.httpx_client = _get_httpx_client(
                 transport_params=self._transport_params,
                 timeout=EMBEDDINGS_HTTPX_TIMEOUT,
             )
+            self._http_client = self._client.httpx_client
+
+    @requests._with_retry(retry_status_codes=_RETRY_STATUS_CODES)
+    def _post(
+        self, http_client: Any, *args: Any, **kwargs: Any
+    ) -> httpx.Response | _requests.Response:
+        return http_client.post(*args, **kwargs)
