@@ -3,16 +3,17 @@
 #  https://opensource.org/licenses/BSD-3-Clause
 #  -----------------------------------------------------------------------------------------
 
-import os
-from typing import Any, Callable
+from typing import Any, Callable, cast
 from warnings import simplefilter, warn
 from pathlib import Path
 import json
+import os
 
 from ibm_watsonx_ai import APIClient, Credentials
 from ibm_watsonx_ai.helpers import DataConnection
 from ibm_watsonx_ai.foundation_models import ModelInference
 from ibm_watsonx_ai.foundation_models.extensions.rag import VectorStore
+
 from ibm_watsonx_ai.foundation_models.extensions.rag.chunker.langchain_chunker import (
     LangChainChunker,
 )
@@ -30,9 +31,27 @@ from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_indexing_fu
 from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service import (
     inference_service as default_inference_service,
 )
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_deployment import (
+    inference_service as default_inference_service_deployment,
+)
 from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_chroma import (
     inference_service as default_inference_service_chroma,
 )
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_chroma_deployment import (
+    inference_service as default_inference_service_chroma_deployment,
+)
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_milvus import (
+    inference_service as default_inference_service_milvus,
+)
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_milvus_deployment import (
+    inference_service as default_inference_service_milvus_deployment,
+)
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_elastic import (
+    inference_service as default_inference_service_elastic,
+)
+from ibm_watsonx_ai.foundation_models.extensions.rag.pattern.default_inference_service_elastic_deployment import (
+    inference_service as default_inference_service_elastic_deployment,
+)
 
 from ibm_watsonx_ai.foundation_models.prompts import PromptTemplateManager
 from ibm_watsonx_ai.foundation_models.utils.enums import PromptTemplateFormats
@@ -279,6 +298,9 @@ class RAGPattern:
 
         self._validate_kwargs()
 
+        # Milvus scenario
+        self._ranker_config = kwargs.get("ranker_config")
+
         if api_client is not None:
             self._credentials = api_client.credentials
             self._client = api_client
@@ -370,8 +392,8 @@ class RAGPattern:
                 "`chunker` param is required when `input_data_references` provided"
             )
 
-        self.indexing_function: RAGPatternFunction | None
-        self.inference_function: RAGPatternFunction | None
+        self._indexing_function: RAGPatternFunction | None
+        self._inference_function: RAGPatternFunction | None
         self.inference_service: RAGPatternService | None
 
         self.deployment_function: RAGPatternFunction | None = None
@@ -381,45 +403,104 @@ class RAGPattern:
             or self._input_data_references is not None
         )
 
-        indexing_function = indexing_function or (
+        indexing_function_tmp = indexing_function or (
             default_indexing_function if self.vector_store and self.chunker else None
         )
-        if indexing_function:
-            self.indexing_function = RAGPatternFunction(
-                api_client=self._client,
-                function=indexing_function,
-                default_params=self._default_indexing_function_params(),
-                store_params=self.store_params,
-                _allow_store=self._allow_store,
-            )
-            if auto_store:
-                self.indexing_function._store_component()
+        self._indexing_function_error = False
+        if indexing_function_tmp:
+            if isinstance(self.vector_store, VectorStore) or indexing_function:
+                self._indexing_function = RAGPatternFunction(
+                    api_client=self._client,
+                    function=indexing_function_tmp,
+                    default_params=self._default_indexing_function_params(),
+                    store_params=self.store_params,
+                    _allow_store=self._allow_store,
+                )
+                if auto_store:
+                    self._indexing_function._store_component()
+            else:
+                self._indexing_function_error = True
+                self._indexing_function = None
         else:
-            self.indexing_function = None
+            self._indexing_function = None
 
         inference_function = inference_function or default_inference_function
+        self._inference_function_error = False
         if inference_custom_asset is None or "function" in inference_custom_asset:
-            self.inference_function = RAGPatternFunction(
-                api_client=self._client,
-                function=inference_function,
-                default_params=self._default_inference_function_params(),
-                store_params=self.store_params,
-                cached=True,
-                _allow_store=self._allow_store,
-            )
-            if auto_store:
-                self.inference_function._store_component()
+            if isinstance(self.vector_store, VectorStore) or (
+                inference_custom_asset is not None
+                and "function" in inference_custom_asset
+            ):
+                self._inference_function = RAGPatternFunction(
+                    api_client=self._client,
+                    function=inference_function,
+                    default_params=self._default_inference_function_params(),
+                    store_params=self.store_params,
+                    cached=False,
+                    _allow_store=self._allow_store,
+                )
+                if auto_store:
+                    self._inference_function._store_component()
+            else:
+                self._inference_function_error = True
+                self._inference_function = None
         else:
-            self.inference_function = None
+            self._inference_function = None
 
         store_params_ai_service: dict | None
 
-        if inference_service is None:
-            inference_service = (
-                default_inference_service_chroma
-                if self._input_data_references is not None
-                else default_inference_service
-            )
+        if inference_service is None and inference_custom_asset is None:
+            self.model = cast(ModelInference, self.model)
+            if self._input_data_references is not None:
+                if self.model.model_id is not None:
+                    inference_service = default_inference_service_chroma
+                else:
+                    inference_service = default_inference_service_chroma_deployment
+            elif self.vector_store:
+                try:
+                    from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                        MilvusVectorStore,
+                    )
+
+                    if isinstance(self.vector_store, MilvusVectorStore):
+                        if self.model.model_id is not None:
+                            inference_service = default_inference_service_milvus
+                        else:
+                            inference_service = (
+                                default_inference_service_milvus_deployment
+                            )
+                except ImportError:
+                    pass
+                if inference_service is None and hasattr(self, "ranker_config"):
+                    raise ValueError(
+                        "`ranker_config` is used only for `MilvusVectorStore`"
+                    )
+                try:
+                    from ibm_watsonx_ai.foundation_models.extensions.rag.vector_stores import (
+                        ElasticsearchVectorStore,
+                    )
+
+                    if isinstance(self.vector_store, ElasticsearchVectorStore):
+                        if self.model.model_id is not None:
+                            inference_service = default_inference_service_elastic
+                        else:
+                            inference_service = (
+                                default_inference_service_elastic_deployment
+                            )
+
+                except ImportError:
+                    pass
+                if inference_service is None:
+                    if self.model.model_id is not None:
+                        inference_service = default_inference_service
+                    else:
+                        inference_service = default_inference_service_deployment
+            else:
+                if self.model.model_id is not None:
+                    inference_service = default_inference_service
+                else:
+                    inference_service = default_inference_service_deployment
+
             store_params_ai_service = self.store_params or {}
             current_dir = Path(__file__).parent
             if (
@@ -456,9 +537,19 @@ class RAGPattern:
             store_params_ai_service = self.store_params
 
         if inference_custom_asset is None or "service" in inference_custom_asset:
-            if (
-                inference_service is default_inference_service
-                or inference_service is default_inference_service_chroma
+            inference_service = cast(Callable, inference_service)
+            if any(
+                inference_service is el
+                for el in (
+                    default_inference_service,
+                    default_inference_service_deployment,
+                    default_inference_service_chroma,
+                    default_inference_service_chroma_deployment,
+                    default_inference_service_milvus,
+                    default_inference_service_milvus_deployment,
+                    default_inference_service_elastic,
+                    default_inference_service_elastic_deployment,
+                )
             ):
                 default_params = self._default_inference_service_params()
             else:
@@ -477,6 +568,40 @@ class RAGPattern:
         else:
             self.inference_service = None
 
+    @property
+    def indexing_function(self) -> RAGPatternFunction | None:
+        """Indexing function object.
+
+        :raises WMLClientError: raise when vector_store is of type different than
+                                 ibm_watsonx_ai.foundation_models.extensions.rag.VectorStore
+
+        :return: indexing function instance
+        :rtype: RAGPatternFunction | None
+        """
+        if self._indexing_function_error:
+            raise WMLClientError(
+                "`indexing_function` not available for the type of provided Vector Store"
+            )
+        else:
+            return self._indexing_function
+
+    @property
+    def inference_function(self) -> RAGPatternFunction | None:
+        """Inference function object.
+
+        :raises WMLClientError: raise when vector_store is of type different than
+                                 ibm_watsonx_ai.foundation_models.extensions.rag.VectorStore
+
+        :return: inference function instance
+        :rtype: RAGPatternFunction | None
+        """
+        if self._inference_function_error:
+            raise WMLClientError(
+                "`inference_function` not available for the type of provided Vector Store"
+            )
+        else:
+            return self._inference_function
+
     def deploy(
         self,
         name: str,
@@ -659,6 +784,7 @@ class RAGPattern:
             "prompt_template_text",
             "context_template_text",
             "default_max_sequence_length",
+            "ranker_config",
         ]
 
         for kwarg in self.kwargs.keys():
@@ -789,6 +915,9 @@ class RAGPattern:
             }
             default_params |= {"chunker": self.chunker.to_dict()}
 
+        if self._ranker_config:
+            default_params["ranker_config"] = self._ranker_config
+
         return default_params
 
     def _default_indexing_function_params(self) -> dict:
