@@ -11,7 +11,6 @@ from typing import (
     overload,
     Literal,
     AsyncGenerator,
-    Any,
 )
 from copy import deepcopy
 
@@ -51,6 +50,9 @@ class DeploymentModelInference(BaseModelInference):
         params: dict | TextChatParameters | TextGenParameters | None = None,
         validate: bool = True,
         persistent_connection: bool = True,
+        max_retries: int | None = None,
+        delay_time: float | None = None,
+        retry_status_codes: list[int] | None = None,
     ) -> None:
         self.deployment_id = deployment_id
 
@@ -73,7 +75,16 @@ class DeploymentModelInference(BaseModelInference):
         if not self._client.CLOUD_PLATFORM_SPACES and self._client.CPD_version < 4.8:
             raise WMLClientError(error_msg="Operation is unsupported for this release.")
 
-        BaseModelInference.__init__(self, __name__, self._client, persistent_connection)
+        BaseModelInference.__init__(
+            self,
+            __name__,
+            self._client,
+            persistent_connection,
+            max_retries,
+            delay_time,
+            retry_status_codes,
+            validate=validate,
+        )
 
     def get_details(self) -> dict:
         """Get deployment's details
@@ -399,6 +410,7 @@ class DeploymentModelInference(BaseModelInference):
             ),
             "project_id": self._client.default_project_id,
             "space_id": self._client.default_space_id,
+            "validate": self._validate,
         }
 
     def _prepare_inference_payload(
@@ -584,6 +596,12 @@ class DeploymentModelInference(BaseModelInference):
                 prompt_template = PromptTemplateManager(
                     api_client=self._client
                 ).load_prompt(prompt_id)
+
+                if not hasattr(
+                    prompt_template, "input_variables"
+                ):  # validate only if it is prompt template
+                    return prompt_required
+
                 prompt_template.input_variables = cast(
                     dict, prompt_template.input_variables
                 )
