@@ -12,7 +12,6 @@ from typing import (
     TYPE_CHECKING,
     overload,
     Literal,
-    Any,
     AsyncGenerator,
 )
 from warnings import catch_warnings, simplefilter
@@ -50,6 +49,9 @@ class FMModelInference(BaseModelInference):
         params: dict | TextChatParameters | TextGenParameters | None = None,
         validate: bool = True,
         persistent_connection: bool = True,
+        max_retries: int | None = None,
+        delay_time: float | None = None,
+        retry_status_codes: list[int] | None = None,
     ):
         self.model_id = model_id
         if isinstance(self.model_id, Enum):
@@ -106,7 +108,16 @@ class FMModelInference(BaseModelInference):
         if not self._client.CLOUD_PLATFORM_SPACES and self._client.CPD_version < 4.8:
             raise WMLClientError(error_msg="Operation is unsupported for this release.")
 
-        BaseModelInference.__init__(self, __name__, self._client, persistent_connection)
+        BaseModelInference.__init__(
+            self,
+            __name__,
+            self._client,
+            persistent_connection,
+            max_retries,
+            delay_time,
+            retry_status_codes,
+            validate=validate,
+        )
 
     def get_details(self) -> dict:
         """Get model's details
@@ -191,7 +202,8 @@ class FMModelInference(BaseModelInference):
             tool_choice_option=tool_choice_option,
         )
 
-        response = await self._async_http_client.post(
+        response = await self._apost(
+            self._async_http_client,
             url=text_chat_url,
             json=payload,
             headers=self._client._get_headers(),
@@ -479,6 +491,7 @@ class FMModelInference(BaseModelInference):
             ),
             "project_id": self._client.default_project_id,
             "space_id": self._client.default_space_id,
+            "validate": self._validate,
         }
 
     def _prepare_inference_payload(  # type: ignore[override]
