@@ -78,9 +78,20 @@ class ModelInference(BaseModelInference):
                                   To close the connection, run `model.close_persistent_connection()`, defaults to True. Added in 1.1.2.
     :type persistent_connection: bool, optional
 
+    :param max_retries: number of retries performed when request was not successful and status code is in retry_status_codes, defaults to 10
+    :type max_retries: int, optional
+
+    :param delay_time: delay time to retry request, factor in exponential backoff formula: wx_delay_time * pow(2.0, attempt), defaults to 0.5s
+    :type delay_time: float, optional
+
+    :param retry_status_codes: list of status codes which will be considered for retry mechanism, defaults to [429, 503, 504, 520]
+    :type retry_status_codes: list[int], optional
+
     .. note::
         * You must provide one of these parameters: [``model_id``, ``deployment_id``]
         * When the ``credentials`` parameter is passed, you must provide one of these parameters: [``project_id``, ``space_id``].
+        * For any "chat" method you can also pass tools from :ref:`Toolkit<fm_toolkit>` converted with
+          :func:`convert_to_watsonx_tool() <ibm_watsonx_ai.foundation_models.utils.convert_to_watsonx_tool>`.
 
     .. hint::
         You can copy the project_id from the Project's Manage tab (Project -> Manage -> General -> Details).
@@ -138,6 +149,9 @@ class ModelInference(BaseModelInference):
         api_client: APIClient | None = None,
         validate: bool = True,
         persistent_connection: bool = True,
+        max_retries: int | None = None,
+        delay_time: float | None = None,
+        retry_status_codes: list[int] | None = None,
     ) -> None:
         self.model_id = model_id
         if isinstance(self.model_id, Enum):
@@ -173,26 +187,27 @@ class ModelInference(BaseModelInference):
                 reason="None of the arguments were provided.",
             )
 
-        if validate or model_id:
-            # 1. When `model_id` is provided, regardless of `validate` value: space, project or api_client is required.
-            # 2. When `deployment_id` is provided and `validate ` is set to True:
-            # space, project or api_client with default project/space is required.
-            if space_id:
-                self._client.set.default_space(space_id)
-            elif project_id:
-                self._client.set.default_project(project_id)
-            elif not (
-                api_client
-                and (
-                    api_client.default_space_id
-                    or api_client.default_project_id
-                    or model_id
-                )
-            ):
-                raise InvalidMultipleArguments(
-                    params_names_list=["space_id", "project_id"],
-                    reason="None of the arguments were provided.",
-                )
+        if space_id:
+            self._client.set.default_space(space_id)
+        elif project_id:
+            self._client.set.default_project(project_id)
+
+        # 1. When `deployment_id` is provided and `validate` is set to True:
+        #    space, project or api_client with default project/space is required.
+        # 2. When `model_id` is provided: space, project or `api_client` is required.
+        #    If `api_client` is provided, we don't check if it has default
+        #    space/project set due to compatibility with lightweight clusters.
+        is_project_or_space_required = bool(
+            (model_id and not api_client) or (deployment_id and validate)
+        )
+        if is_project_or_space_required and not (
+            self._client.default_project_id or self._client.default_space_id
+        ):
+            raise InvalidMultipleArguments(
+                params_names_list=["space_id", "project_id"],
+                reason="None of the arguments were provided.",
+            )
+
         if not self._client.CLOUD_PLATFORM_SPACES and self._client.CPD_version < 4.8:
             raise WMLClientError(error_msg="Operation is unsupported for this release.")
 
@@ -204,6 +219,9 @@ class ModelInference(BaseModelInference):
                 params=self.params,
                 validate=validate,
                 persistent_connection=persistent_connection,
+                max_retries=max_retries,
+                delay_time=delay_time,
+                retry_status_codes=retry_status_codes,
             )
         else:
             self.deployment_id = cast(str, self.deployment_id)
@@ -213,6 +231,9 @@ class ModelInference(BaseModelInference):
                 params=self.params,
                 validate=validate,
                 persistent_connection=persistent_connection,
+                max_retries=max_retries,
+                delay_time=delay_time,
+                retry_status_codes=retry_status_codes,
             )
 
         self._transport_params = requests._httpx_transport_params(self._client)
@@ -1008,14 +1029,20 @@ class ModelInference(BaseModelInference):
         return self._inference.get_identifying_params()
 
     def close_persistent_connection(self) -> None:
-        """Only applicable if persistent_connection was set to True in ModelInference initialization."""
+        """
+        Only applicable if persistent_connection was set to True in ModelInference initialization.
+        Calling this method closes the current `httpx.Client` and recreates a new `httpx.Client` with default values:
+        timeout: httpx.Timeout(timeout=30 * 60, connect=10)
+        limit: httpx.Limits(max_connections=10, max_keepalive_connections=10, keepalive_expiry=HTTPX_KEEPALIVE_EXPIRY)
+        """
         if self._inference._persistent_connection and isinstance(
             self._inference._http_client, httpx.Client
         ):
             self._inference._http_client.close()
-            self._inference._http_client = _get_httpx_client(
+            self._client.httpx_client = _get_httpx_client(
                 transport_params=self._transport_params
             )
+            self._inference._http_client = self._client.httpx_client
 
     def set_api_client(self, api_client: APIClient) -> None:
         """
@@ -1085,6 +1112,7 @@ class ModelInference(BaseModelInference):
             self._inference._async_http_client, httpx.AsyncClient
         ):
             await self._inference._async_http_client.aclose()
-            self._inference._async_http_client = _get_async_client(
+            self._client.async_httpx_client = _get_async_client(
                 transport_params=self._transport_params
             )
+            self._inference._async_http_client = self._client.async_httpx_client
