@@ -7,6 +7,12 @@ from ibm_watsonx_ai.foundation_models.utils.utils import (
     PromptTuningParams,
     FineTuningParams,
 )
+from ibm_watsonx_ai.foundation_models.utils.toolkit import (
+    Toolkit,
+    Tool,
+    convert_to_watsonx_tool,
+    convert_to_utility_tool_call,
+)
 from ibm_watsonx_ai.foundation_models.utils.utils import HAPDetectionWarning
 from ibm_watsonx_ai.foundation_models.utils.utils import (
     get_model_specs,
