@@ -25,10 +25,12 @@ import io
 import os
 import sys
 import uuid
+import re
+
 from copy import deepcopy
 from typing import Union, Tuple, List, TYPE_CHECKING, Optional, Any
+from pathlib import Path
 from warnings import warn
-import re
 
 import numpy as np
 from ibm_boto3 import resource
@@ -49,7 +51,7 @@ from ibm_watsonx_ai.utils.autoai.errors import (
 from ibm_watsonx_ai.utils.autoai.errors import (
     MissingAutoPipelinesParameters,
     MissingCOSStudioConnection,
-    MissingProjectLib,
+    MissingIBMWatsonStudioLib,
     InvalidCOSCredentials,
     InvalidIdType,
     NotExistingCOSResource,
@@ -275,51 +277,56 @@ class DataConnection(BaseDataConnection):
 
             data_connections = DataConnection.from_studio(path='iris_dataset.csv')
         """
+
+        from_studio_deprecation_warning = (
+            "`DataConnection.from_studio` is deprecated and will be removed in future. "
+            "Instead, initialize data connections directly or use `DataConnection.from_dict`."
+        )
+        warn(from_studio_deprecation_warning, category=DeprecationWarning)
+
         try:
-            from project_lib import Project
-
+            from ibm_watson_studio_lib.impl.agent import Agent
         except ModuleNotFoundError:
-            raise MissingProjectLib("Missing project_lib package.")
+            raise MissingIBMWatsonStudioLib("Missing ibm-watson-studio-lib package.")
 
-        else:
-            data_connections = []
-            for name, value in globals().items():
-                if isinstance(value, Project):
-                    connections = value.get_connections()
+        data_connections = []
+        for value in globals().values():
+            if not (
+                isinstance(value, Agent) and (connections := value.list_connections())
+            ):
+                continue
 
-                    if connections:
-                        for connection in connections:
-                            asset_id = connection["asset_id"]
-                            connection_details = value.get_connection(asset_id)
+            for connection in connections:
+                asset_id = connection["asset_id"]
+                connection_details = value.by_id.get_connection(asset_id)
 
-                            if (
-                                "url" in connection_details
-                                and "access_key" in connection_details
-                                and "secret_key" in connection_details
-                                and "bucket" in connection_details
-                            ):
-                                data_connections.append(
-                                    cls(
-                                        connection=ConnectionAsset(
-                                            connection_id=connection_details["id"]
-                                        ),
-                                        location=ConnectionAssetLocation(
-                                            bucket=connection_details["bucket"],
-                                            file_name=path,
-                                        ),
-                                    )
-                                )
+                if any(
+                    key not in connection_details
+                    for key in ["url", "access_key", "secret_key", "bucket"]
+                ):
+                    continue
 
-            if data_connections:
-                return data_connections
-
-            else:
-                raise MissingCOSStudioConnection(
-                    "There is no any COS Studio connection. "
-                    "Please create a COS connection from the UI and insert "
-                    "the cell with project API connection (Insert project token)"
+                data_connections.append(
+                    cls(
+                        connection=ConnectionAsset(
+                            connection_id=asset_id,
+                        ),
+                        location=ConnectionAssetLocation(
+                            bucket=connection_details["bucket"],
+                            file_name=path,
+                        ),
+                    )
                 )
 
+        if not data_connections:
+            raise MissingCOSStudioConnection(
+                "There is no any COS Studio connection. "
+                "Please create a COS connection from the UI and insert "
+                "the cell with project API connection (Insert project token)"
+            )
+
+        return data_connections
+
     def _prepare_flight_connection_for_discovery(self) -> "FlightConnection":
         """Returns initialized FlightClient that can be used to call discovery or any other action
 
@@ -355,45 +362,23 @@ class DataConnection(BaseDataConnection):
 
         return conn
 
-    def _get_file_paths_from_nfs_folder(self) -> list[str]:
-        """Returns file paths (keys) of objects that are stored at a NFS/ Storage Volume location.
+    def _get_paths_from_location(self, include_folders: bool = False) -> list[str]:
+        """Returns file and folder (optional) paths (keys) of objects that are stored at a NFS / Storage Volume or bucket location.
 
-        Returns all file paths under the ``self.location.get_location()`` prefix.
+        The following 'location' types are supported: `S3Location`, `ContainerLocation`, `NFSLocation`.
+
+        Returns all file and folder (optional) paths under the ``self.location.get_location()`` prefix.
         First checks if the prefix exists as a bucket "directory" - if not, treats prefix as the file name.
 
-        :return: list of file names (keys) of objects in a bucket location
-        :rtype: list[str]
-        """
+        :param include_folders: if `True` - folder paths are included, defaults to `False`
+        :type include_folders: bool, optional
 
-        prefix = self.location.get_location().strip("/")
-
-        try:
-            flight_conn = self._prepare_flight_connection_for_discovery()
-
-            paths = [
-                r["path"]
-                for r in flight_conn.discovery(f"{prefix}")["assets"]
-                if r["type"] == "file"
-            ]
-        except Exception as e:
-            raise WMLClientError(
-                f"Could not retrieve file paths from connection. Reason: {e}"
-            )
-        else:
-            if paths:
-                return paths
-            else:
-                raise InvalidLocationInDataConnection(self.location.get_location())
-
-    def _get_file_paths_from_bucket(self) -> list[str]:
-        """Returns file paths (keys) of objects that are stored at a bucket location.
-
-        Returns all file paths under the ``self.location.get_location()`` prefix.
-        First checks if the prefix exists as a bucket "directory" - if not, treats prefix as the file name.
-
-        :return: list of file names (keys) of objects in a bucket location
+        :return: list of paths to objects at NFS or bucket location
         :rtype: list[str]
         """
+        include_types = {"file"}
+        if include_folders:
+            include_types.add("folder")
 
         prefix = self.location.get_location().strip("/")
 
@@ -405,10 +390,14 @@ class DataConnection(BaseDataConnection):
                 for r in flight_conn.discovery(f"/{self.location.bucket}/{prefix}")[
                     "assets"
                 ]
-                if r["type"] == "file"
+                if r["type"] in include_types
             ]
 
         except Exception as e:
+            if include_folders or isinstance(self.location, NFSLocation):
+                raise WMLClientError(
+                    f"Could not retrieve paths from connection. Reason: {e}"
+                )
             warn(f"Flight discovery didn't work, error: {e}")
             return self._get_file_paths_from_bucket_fallback()
         else:
@@ -458,11 +447,16 @@ class DataConnection(BaseDataConnection):
 
         return get_keys_with_prefix(bucket_objects, prefix)
 
-    def _get_connections_from_folder(self) -> list["DataConnection"]:
-        """Return connections for every object in a bucket location.
+    def _get_connections_from_folder(
+        self, include_folders: bool = False
+    ) -> list["DataConnection"]:
+        """Return connections for every file and folder (optional) in a bucket / NFS location.
 
         :raises WMLClientError: If location is not one of ``S3Location``, ``ContainerLocation``, ``NFSLocation``
 
+        :param include_folders: if `True` - connection for folders are included, defaults to `False`
+        :type include_folders: bool, optional
+
         :return: list of connections to objects in a bucket location
         :rtype: list[DataConnection]
         """
@@ -478,29 +472,25 @@ class DataConnection(BaseDataConnection):
 
         new_data_connections = []
 
-        if isinstance(self.location, NFSLocation):
-            file_paths = self._get_file_paths_from_nfs_folder()
+        if isinstance(self.location, ContainerLocation):
+            self._check_if_connection_asset_is_s3()
 
-        else:
-            if isinstance(self.location, ContainerLocation):
-                self._check_if_connection_asset_is_s3()
+        paths = self._get_paths_from_location(include_folders)
 
-            file_paths = self._get_file_paths_from_bucket()
-
-        for file_path in file_paths:
+        for path in paths:
             if isinstance(self.location, NFSLocation):
                 new_data_conn = DataConnection(
                     connection=self.connection,
-                    location=NFSLocation(path=file_path),
+                    location=NFSLocation(path=path),
                 )
             elif isinstance(self.location, ContainerLocation):
                 new_data_conn = DataConnection(
-                    location=ContainerLocation(path=file_path),
+                    location=ContainerLocation(path=path),
                 )
             else:
                 new_data_conn = DataConnection(
                     connection=self.connection,
-                    location=S3Location(bucket=self.location.bucket, path=file_path),
+                    location=S3Location(bucket=self.location.bucket, path=path),
                 )
             if self._api_client:
                 new_data_conn.set_client(self._api_client)
@@ -1099,8 +1089,7 @@ class DataConnection(BaseDataConnection):
         if self.type == DataConnectionTypes.DS or self.type == DataConnectionTypes.CA:
             if self._api_client is None:
                 try:
-                    from project_lib import Project
-
+                    import ibm_watson_studio_lib
                 except ModuleNotFoundError:
                     raise ConnectionError(
                         "This functionality can be run only on Watson Studio or with api_client passed to connection. "
@@ -1177,7 +1166,7 @@ class DataConnection(BaseDataConnection):
                         if not flight_parameters.get("connection_properties"):
                             flight_parameters = (
                                 self._update_flight_parameters_with_connection_details(
-                                    flight_parameters
+                                    flight_parameters, binary
                                 )
                             )
 
@@ -1298,7 +1287,7 @@ class DataConnection(BaseDataConnection):
                     if not flight_parameters.get("connection_properties"):
                         flight_parameters = (
                             self._update_flight_parameters_with_connection_details(
-                                flight_parameters
+                                flight_parameters, binary
                             )
                         )
 
@@ -1763,33 +1752,62 @@ class DataConnection(BaseDataConnection):
         except Exception as e:
             raise NotExistingCOSResource(self.location.bucket, self.location.path)
 
-    def _update_flight_parameters_with_connection_details(self, flight_parameters):
+    def _update_flight_parameters_with_connection_details(
+        self, flight_parameters: dict, is_binary=True
+    ):
         with all_logging_disabled():
             self._check_if_connection_asset_is_s3()
-            connection_properties = {
-                "bucket": self.location.bucket,
-                "url": self.connection.endpoint_url,
-            }
-            if hasattr(self.connection, "auth_endpoint") and hasattr(
-                self.connection, "api_key"
-            ):
-                connection_properties["iam_url"] = self.connection.auth_endpoint
-                connection_properties["api_key"] = self.connection.api_key
-                connection_properties["resource_instance_id"] = (
-                    self.connection.resource_instance_id
-                )
 
-            else:
-                connection_properties["secret_key"] = self.connection.secret_access_key
-                connection_properties["access_key"] = self.connection.access_key_id
+        connection_properties = {
+            "bucket": self.location.bucket,
+            "url": self.connection.endpoint_url,
+        }
 
-            flight_parameters.update({"connection_properties": connection_properties})
-            flight_parameters.update(
-                {"datasource_type": {"entity": {"name": self._datasource_type}}}
+        if all(hasattr(self.connection, key) for key in ["auth_endpoint", "api_key"]):
+            connection_properties["iam_url"] = self.connection.auth_endpoint
+            connection_properties["api_key"] = self.connection.api_key
+            connection_properties["resource_instance_id"] = (
+                self.connection.resource_instance_id
             )
+        else:
+            connection_properties["secret_key"] = self.connection.secret_access_key
+            connection_properties["access_key"] = self.connection.access_key_id
+
+        flight_parameters["connection_properties"] = connection_properties
+        flight_parameters["datasource_type"] = {
+            "entity": {"name": self._datasource_type}
+        }
+
+        if (file_format := self._get_file_format(is_binary)) is not None:
+            flight_parameters["interaction_properties"] = {"file_format": file_format}
+
+            if file_format in {"csv", "delimited", "excel"}:
+                flight_parameters["interaction_properties"]["first_line_header"] = True
 
         return flight_parameters
 
+    def _get_file_format(self, is_binary: bool) -> str | None:
+        if is_binary:
+            return None
+
+        try:
+            filename = Path(self._get_filename())
+
+            # For MS Excel files, the path can have the sheet name concatenated
+            # at the end, like /path/to/file/file.xls(x)/sheet-name, so the
+            # parent's extension needs to be checked.
+            extension = filename.suffix or filename.parent.suffix
+        except (CannotGetFilename, DirectoryHasNoFilename, TypeError):
+            return None
+
+        match extension.lower():
+            case ".csv":
+                return "csv"
+            case ".xlsx" | ".xls":
+                return "excel"
+            case _:
+                return None
+
     def download(self, filename: str) -> None:
         """Download a dataset stored in a remote data storage and save to a file.
 
@@ -1810,6 +1828,50 @@ class DataConnection(BaseDataConnection):
         with open(filename, "wb") as file:
             file.write(self.read(binary=True))
 
+    def download_folder(self, local_dir: str | None = None) -> None:
+        """Download files from a folder and subfolders stored in a remote data storage and save to a local directory.
+
+        :param local_dir: path to the local directory where data will be downloaded, download to current working directory if not provided
+        :type local_dir: str, optional
+
+        **Examples**
+
+        .. code-block:: python
+
+            folder_reference = DataConnection(
+                connection_asset_id="<connection_id>",
+                location=S3Location(bucket="<bucket_name>", path="path/to/folder"),
+                )
+            folder_reference.download(local_dir="./data")
+
+        """
+        if not isinstance(self.location, (S3Location, ContainerLocation, NFSLocation)):
+            raise WMLClientError(
+                error_msg="Can't download folder from this DataConnection.",
+                reason="Location must be one of: `S3Location`, `ContainerLocation`, `NFSLocation`.",
+            )
+
+        if local_dir is None:
+            local_dir = os.getcwd()
+        else:
+            os.makedirs(local_dir, exist_ok=True)
+
+        file_extension = os.path.splitext(self.location.get_location())[1]
+        if file_extension:
+            raise WMLClientError(
+                "Location of the data connection does not point to a folder."
+            )
+
+        data_connections = self._get_connections_from_folder(include_folders=True)
+        for data_connection in data_connections:
+            extension = os.path.splitext(data_connection.location.get_location())[1]
+            item_name = os.path.basename(data_connection.location.get_location())
+            path = os.path.join(local_dir, item_name)
+            if extension:
+                data_connection.download(path)
+            else:
+                data_connection.download_folder(path)
+
     def _get_filename(self):
         """Get file name of the file in data connection, if applicable.
 
