@@ -10,6 +10,7 @@ import time
 import threading
 import queue
 import pandas as pd
+from random import random
 
 from ast import literal_eval
 from contextlib import nullcontext
@@ -39,7 +40,6 @@ from warnings import warn
 from math import ceil
 from ibm_watsonx_ai.wml_client_error import WMLClientError
 
-
 logger = logging.getLogger(__name__)
 
 DEFAULT_PARTITIONS_NUM = 4
@@ -161,6 +161,9 @@ class FlightConnection:
 
     :param apply_literal_eval: when True then ast.literal_eval will be applied to all string columns.
     :type apply_literal_eval: bool, optional
+
+    :param max_retry_time: maximal time for retrying in seconds (the whole retrying process should take less than max_retry_time)
+    :type max_retry_time: int, optional
     """
 
     def __init__(
@@ -189,6 +192,7 @@ class FlightConnection:
         total_nrows_limit=None,
         total_percentage_limit=1.0,
         apply_literal_eval=False,
+        max_retry_time: int = 200,
         **kwargs,
     ) -> None:
 
@@ -250,6 +254,7 @@ class FlightConnection:
 
         # callback is used in the backend to send status messages
         self.callback = callback if callback is not None else FakeCallback()
+        self.max_retry_time = max_retry_time
 
         # Handle logical_batch_size_limit parameter set up
         if logical_batch_size_limit is None:
@@ -464,33 +469,23 @@ class FlightConnection:
 
     def get_endpoints(self) -> Iterable[List["flight.FlightEndpoint"]]:
         """Listing all available Flight Service endpoints (one endpoint corresponds to one batch)"""
-
-        max_auth_waiting_time = 180
-        real_waiting_time = 0
-        count_auth_retries_power = 4
-        retry_authentication = True
-
-        while retry_authentication:
-            try:
-                retry_authentication = False
-                if exp_time := _get_expiration_datetime_from_headers(self.headers):
-                    logger.debug(f"Token expiration time: {exp_time}")
-            except Exception as e:
-                if "failed to connect to all addresses" in str(e):
-                    retry_authentication = True  # wait up to 180 seconds - Flight Service can be restarting issue #30564
-                else:
-                    retry_authentication = False
-
-                if retry_authentication and real_waiting_time < max_auth_waiting_time:
-                    logger.debug(
-                        f"Cannot connect to Flight Service in {real_waiting_time}s,"
-                        f" attempting to retry the authentication. "
+        max_retries = 3
+        for source_command in self._select_source_command():
+            last_exception = None
+            attempt = 0
+            while attempt < max_retries + 1:
+                prolong = 0
+                try:
+                    info = self.flight_client.get_flight_info(
+                        flight.FlightDescriptor.for_command(source_command)
                     )
-                    waiting_time = 2**count_auth_retries_power
-                    real_waiting_time += waiting_time
-                    time.sleep(waiting_time)
-                    count_auth_retries_power += 1
-                else:
+                    yield info.endpoints
+                    return
+
+                except flight.FlightUnauthenticatedError as e:
+                    last_exception = e
+                    if exp_time := _get_expiration_datetime_from_headers(self.headers):
+                        logger.debug(f"Token expiration time: {exp_time}")
                     # suggest CPD users to check the Flight variables
                     if hasattr(self._api_client, "ICP_PLATFORM_SPACES"):
                         if self._api_client.ICP_PLATFORM_SPACES:
@@ -507,73 +502,64 @@ class FlightConnection:
                         raise ConnectionError(
                             f"Cannot connect to the Flight service. Error: {e}"
                         )
-
-        count = 0
-        retry = True
-        while retry:
-            try:
-                retry = False
-                count += 1
-                for source_command in self._select_source_command():
-                    info = self.flight_client.get_flight_info(
-                        flight.FlightDescriptor.for_command(source_command)
+                except flight.FlightInternalError as e:
+                    last_exception = e
+                    logger.debug(
+                        f"Caught FlightInternalError in get_endpoints: {str(e)}"
                     )
-
-                    yield info.endpoints
-
-            except flight.FlightInternalError as e:
-                logger.debug(f"Caught FlightInternalError in get_endpoints: {str(e)}")
-                if "CDICO2034E" in str(e):
-                    if "The property [infer_as_varchar] is not supported." in str(e):
-                        # Don't send infer_as_varchar in flight command and try again.
-                        self.infer_as_varchar = None
-                        retry = True
-                    elif "The property [quote_character]" in str(e):
-                        # Don't send quote_character in flight command if it is not yet supported and try again.
-                        self.params["quote_character"] = None
-                        retry = True
-                    else:
+                    if "CDICO2034E" in str(e):
+                        if "The property [infer_as_varchar] is not supported." in str(
+                            e
+                        ):
+                            # Don't send infer_as_varchar in flight command and try again.
+                            self.infer_as_varchar = None
+                        elif "The property [quote_character]" in str(e):
+                            # Don't send quote_character in flight command if it is not yet supported and try again.
+                            self.params["quote_character"] = None
+                        else:
+                            raise WrongLocationProperty(reason=str(e))
+                    elif "CDICO2015E" in str(e):
+                        raise WrongFileLocation(reason=str(e))
+                    elif "CDICO9999E" in str(e):
                         raise WrongLocationProperty(reason=str(e))
+                    elif not any(
+                        code in str(e)
+                        for code in ["CDICO2016E", "CDICO2026E", "CDICO2027E"]
+                    ):
+                        raise e
 
-                    if count > 1:
-                        logger.debug(f"Reached the max retry times ${count}")
+                except pa.lib.ArrowInvalid as e:
+                    last_exception = e
+                    logger.debug(f"Caught ArrowInvalid in get_endpoints: {str(e)}")
+                    if not any(
+                        keyword in str(e)
+                        for keyword in ["No asset found with the id", "Asset with ID"]
+                    ):
                         raise e
-                elif any(
-                    err_code in str(e)
-                    for err_code in ["CDICO2016E", "CDICO2026E", "CDICO2027E"]
-                ):
-                    retry = True
-                    if count < 3:
+
+                except Exception as e:
+                    last_exception = e
+                    if "failed to connect to all addresses" in str(e):
                         logger.debug(
-                            f"Retry to get the endpoints of flight service at the ${count} time"
+                            f"Cannot connect to Flight Service. "
+                            f"Flight Service can be restarting, prolongation sleeping time to 180s"
                         )
-                        time.sleep(3)
+                        # wait up to 180 seconds - Flight Service can be restarting issue #30564
+                        prolong = 180
                     else:
-                        logger.debug(f"Reached the max retry times ${count}")
                         raise e
-                elif "CDICO2015E" in str(e):
-                    raise WrongFileLocation(reason=str(e))
-                elif "CDICO9999E" in str(e):
-                    raise WrongLocationProperty(reason=str(e))
-                else:
-                    raise e
-            except pa.lib.ArrowInvalid as e:
-                logger.debug(f"Caught ArrowInvalid in get_endpoints: {str(e)}")
-                if any(
-                    err_msg in str(e)
-                    for err_msg in ["No asset found with the id", "Asset with ID"]
-                ):
-                    retry = True
-                    if count < 3:
-                        logger.debug(
-                            f"Retry to get the endpoints of flight service at the ${count} time"
-                        )
-                        time.sleep(3)
-                    else:
-                        logger.debug(f"Reached the max retry times ${count}")
-                        raise e
-                else:
-                    raise e
+
+                logger.debug(f"Retry {attempt + 1} of {max_retries} in progress.")
+
+                jitter = 1 + 0.25 * random()
+                sleep_seconds = min(
+                    pow(2.0, attempt) * jitter + prolong, self.max_retry_time
+                )
+                time.sleep(sleep_seconds)
+                attempt += 1
+
+            logger.debug(f"Maximum retries reached: {attempt}/{max_retries}.")
+            raise last_exception
 
     def _get_data(self, thread_number: int, endpoint: "flight.FlightEndpoint") -> None:
         """Read data from Flight Service (only one batch).
@@ -1012,8 +998,8 @@ class FlightConnection:
                 self.yielded_nrows += len(self.data)
                 yield self.data
 
-        except InvalidSamplingType as sampling_error:
-            raise sampling_error
+        except (InvalidSamplingType, WrongFileLocation) as e:
+            raise e
 
         except ValueError as e:
             if not self.fallback_to_one_connection:
@@ -1377,25 +1363,18 @@ class FlightConnection:
                 )
             )
 
-            if "file_format" in command["interaction_properties"] and (
-                "write_mode" in command["interaction_properties"] or self.read_binary
-            ):
-                del command["interaction_properties"]["file_format"]
+            if "write_mode" in command["interaction_properties"] or self.read_binary:
+                if "file_format" in command["interaction_properties"]:
+                    del command["interaction_properties"]["file_format"]
 
-            if "sheet_name" in command["interaction_properties"] and (
-                "write_mode" in command["interaction_properties"] or self.read_binary
-            ):
-                del command["interaction_properties"]["sheet_name"]
+                if "sheet_name" in command["interaction_properties"]:
+                    del command["interaction_properties"]["sheet_name"]
 
-            if "encoding" in command["interaction_properties"] and (
-                "write_mode" in command["interaction_properties"] or self.read_binary
-            ):
-                del command["interaction_properties"]["encoding"]
+                if "encoding" in command["interaction_properties"]:
+                    del command["interaction_properties"]["encoding"]
 
-            if "quote_character" in command["interaction_properties"] and (
-                "write_mode" in command["interaction_properties"] or self.read_binary
-            ):
-                del command["interaction_properties"]["quote_character"]
+                if "quote_character" in command["interaction_properties"]:
+                    del command["interaction_properties"]["quote_character"]
 
             if infer_schema:
                 command["interaction_properties"]["infer_schema"] = "true"
@@ -1599,7 +1578,7 @@ class FlightConnection:
 
     def _sample_data_to_percentage(self, percentage, data, full_data_nrows):
         if self.enable_subsampling:
-            if percentage < 1.0 and percentage > 0.0:
+            if 1.0 > percentage > 0.0:
                 nrows = int(percentage * full_data_nrows)
                 logger.debug(
                     f"Running sampling to prercentage of data: "
@@ -1638,14 +1617,17 @@ class FlightConnection:
 
     def data_dropna(self, data, label_column):
         # simple preprocess before sampling
-        data_with_nans_len = len(data)
-        data.dropna(inplace=True, subset=[label_column])
-        removed_rows = data_with_nans_len - len(data)
-        if removed_rows > 0:
-            self.downloaded_data_nrows = (
-                self.downloaded_data_nrows - removed_rows
-            )  # fix for no_batches calculation #31307
+        try:
+            data_with_nans_len = len(data)
+            data.dropna(inplace=True, subset=[label_column])
+            removed_rows = data_with_nans_len - len(data)
+            if removed_rows > 0:
+                self.downloaded_data_nrows = (
+                    self.downloaded_data_nrows - removed_rows
+                )  # fix for no_batches calculation #31307
 
+        except KeyError:
+            warn(f"Error dropping nan values from column {label_column}")
         return data
 
     @staticmethod
