@@ -99,6 +99,13 @@ SPACE_PLATFORM_HREF_PATTERN = "{}/v2/spaces/{}"
 SPACES_MEMBERS_HREF_PATTERN = "{}/v2/spaces/{}/members"
 SPACES_MEMBER_HREF_PATTERN = "{}/v2/spaces/{}/members/{}"
 
+PROJECT = "{}/v2/projects/{}"
+PROJECTS = "{}/v2/projects"
+TRANSACTIONAL_PROJECT = "{}/transactional/v2/projects/{}"
+TRANSACTIONAL_PROJECTS = "{}/transactional/v2/projects"
+PROJECTS_MEMBERS_HREF_PATTERN = "{}/v2/projects/{}/members"
+PROJECTS_MEMBER_HREF_PATTERN = "{}/v2/projects/{}/members/{}"
+
 V4_INSTANCE_ID_HREF_PATTERN = "{}/ml/v4/instances/{}"
 
 API_VERSION = "/v4"
@@ -113,6 +120,10 @@ ASSET = "{}/v2/assets/{}"
 ASSETS = "{}/v2/assets"
 ASSET_TYPE = "{}/v2/asset_types"
 ASSET_FILES = "{}/v2/asset_files/"
+TRASHED_ASSETS = "{}/v2/trashed_assets"
+TRASHED_ASSETS_PURGE_ALL = "{}/v2/trashed_assets/purge_all"
+TRASHED_ASSET = "{}/v2/trashed_assets/{}"
+TRASHED_ASSET_RESTORE = "{}/v2/trashed_assets/{}/restore"
 ATTACHMENT = "{}/v2/assets/{}/attachments/{}"
 ATTACHMENT_COMPLETE = "{}/v2/assets/{}/attachments/{}/complete"
 ATTACHMENTS = "{}/v2/assets/{}/attachments"
@@ -150,8 +161,7 @@ PACKAGE_EXTENSION = "{}/v2/package_extensions/{}"
 PACKAGE_EXTENSIONS = "{}/v2/package_extensions"
 PARAMETER_SET = "{}/v2/parameter_sets/{}"
 PARAMETER_SETS = "{}/v2/parameter_sets"
-PROJECT = "{}/v2/projects/{}"
-LIST_SOFTWARE_SPECIFICATIONS = "{}/ml/v4/list/software_specifications"
+JOBS_RUNS = "{}/v2/jobs/{}/runs/{}"
 
 V4GA_CLOUD_MIGRATION = "{}/ml/v4/repository"
 V4GA_CLOUD_MIGRATION_ID = "{}/ml/v4/repository/{}"
@@ -192,7 +202,10 @@ PROMPTS = "{}/wx/v1-beta/prompts"
 GA_PROMPTS = "{}/wx/v1/prompts"
 PROMPTS_GET_ALL = "{}/v2/asset_types/wx_prompt/search"
 
-TEXT_EXTRACTION = "{}/ml/v1/text/extractions"
+TEXT_DETECTION = "{}/ml/v1/text/detection"
+
+TEXT_EXTRACTIONS = "{}/ml/v1/text/extractions"
+TEXT_EXTRACTION = "{}/ml/v1/text/extractions/{}"
 
 RERANK = "{}/ml/v1/text/rerank"
 
@@ -217,8 +230,7 @@ WKC_MODEL_LIST_ALL = "{}/v1/aigov/model_inventory/model_entries"
 TASK_CREDENTIALS = "{}/v1/task_credentials/{}"
 TASK_CREDENTIALS_ALL = "{}/v1/task_credentials"
 
-WX_TAXONOMY = "{}/v2/assets/{}/attributes/wx_taxonomy"
-GIT_BASED_WX_TAXONOMY = "{}/userfs/v2/assets/{}/attributes/wx_taxonomy"
+TAXONOMY = "{}/ml/v4/taxonomies/{}"
 TAXONOMIES_IMPORTS = "{}/ml/v1/tuning/taxonomies_imports"
 TAXONOMIES_IMPORT = "{}/ml/v1/tuning/taxonomies_imports/{}"
 DOCUMENT_EXTRACTIONS = "{}/ml/v1/tuning/documents"
@@ -226,6 +238,9 @@ DOCUMENT_EXTRACTION = "{}/ml/v1/tuning/documents/{}"
 SYNTHETIC_DATA_GENERATIONS = "{}/ml/v1/tuning/synthetic_data"
 SYNTHETIC_DATA_GENERATION = "{}/ml/v1/tuning/synthetic_data/{}"
 
+UTILITY_AGENT_TOOLS_BETA = "{}/wx/v1-beta/utility_agent_tools"
+UTILITY_AGENT_TOOLS_RUN_BETA = "{}/wx/v1-beta/utility_agent_tools/run"
+
 
 def is_url(s: str) -> bool:
     res = re.match(r"https?:\/\/.+", s)
@@ -428,6 +443,16 @@ class HrefDefinitions:
             self._get_platform_url_if_exists(), spaces_id
         )
 
+    def get_projects_member_href(self, project_id: str, member_id: str) -> str:
+        return PROJECTS_MEMBER_HREF_PATTERN.format(
+            self._get_platform_url_if_exists(), project_id, member_id
+        )
+
+    def get_projects_members_href(self, project_id: str) -> str:
+        return PROJECTS_MEMBERS_HREF_PATTERN.format(
+            self._get_platform_url_if_exists(), project_id
+        )
+
     def get_v4_instance_id_href(self, instance_id: str) -> str:
         return V4_INSTANCE_ID_HREF_PATTERN.format(self._credentials.url, instance_id)
 
@@ -569,6 +594,18 @@ class HrefDefinitions:
             else GIT_BASED_PROJECT_ASSET_TYPE
         ).format(self._credentials.url)
 
+    def get_trashed_assets_href(self) -> str:
+        return TRASHED_ASSETS.format(self._credentials.url)
+
+    def get_trashed_assets_purge_all_href(self) -> str:
+        return TRASHED_ASSETS_PURGE_ALL.format(self._credentials.url)
+
+    def get_trashed_asset_href(self, asset_id: str) -> str:
+        return TRASHED_ASSET.format(self._credentials.url, asset_id)
+
+    def get_trashed_asset_restore_href(self, asset_id: str) -> str:
+        return TRASHED_ASSET_RESTORE.format(self._credentials.url, asset_id)
+
     def get_connections_href(self) -> str:
         return CONNECTION_ASSET.format(self._get_platform_url_if_exists())
 
@@ -616,8 +653,16 @@ class HrefDefinitions:
     def get_project_href(self, project_id: str) -> str:
         return PROJECT.format(self._get_platform_url_if_exists(), project_id)
 
-    def get_software_specifications_list_href(self) -> str:
-        return LIST_SOFTWARE_SPECIFICATIONS.format(self._credentials.url)
+    def get_projects_href(self) -> str:
+        return PROJECTS.format(self._get_platform_url_if_exists())
+
+    def get_transactional_project_href(self, project_id: str) -> str:
+        return TRANSACTIONAL_PROJECT.format(
+            self._get_platform_url_if_exists(), project_id
+        )
+
+    def get_transactional_projects_href(self) -> str:
+        return TRANSACTIONAL_PROJECTS.format(self._get_platform_url_if_exists())
 
     def v4ga_cloud_migration_href(self) -> str:
         return V4GA_CLOUD_MIGRATION.format(self._credentials.url)
@@ -775,8 +820,14 @@ class HrefDefinitions:
             self._get_platform_url_if_exists()
         )
 
-    def get_text_extraction_href(self, ga_api: bool = True) -> str:
-        return TEXT_EXTRACTION.format(self._credentials.url)
+    def get_text_detection_href(self) -> str:
+        return TEXT_DETECTION.format(self._credentials.url)
+
+    def get_text_extractions_href(self) -> str:
+        return TEXT_EXTRACTIONS.format(self._credentials.url)
+
+    def get_text_extraction_href(self, text_extraction_id: str) -> str:
+        return TEXT_EXTRACTION.format(self._credentials.url, text_extraction_id)
 
     def get_rerank_href(self) -> str:
         return RERANK.format(self._credentials.url)
@@ -810,12 +861,10 @@ class HrefDefinitions:
     def get_time_series_href(self) -> str:
         return FM_TIME_SERIES.format(self._credentials.url)
 
-    def get_wx_taxonomy_href(self, taxonomy_id: str) -> str:
-        return (
-            WX_TAXONOMY if not self._is_git_based_project() else GIT_BASED_WX_TAXONOMY
-        ).format(self._get_platform_url_if_exists(), taxonomy_id)
+    def get_taxonomy_href(self, taxonomy_id: str) -> str:
+        return TAXONOMY.format(self._credentials.url, taxonomy_id)
 
-    def get_taxonomies_imports_href(self):
+    def get_taxonomies_imports_href(self) -> str:
         return TAXONOMIES_IMPORTS.format(self._credentials.url)
 
     def get_taxonomies_import_href(self, taxonomy_import_id: str):
@@ -832,3 +881,12 @@ class HrefDefinitions:
 
     def get_synthetic_data_generation_href(self, sdg_id: str):
         return SYNTHETIC_DATA_GENERATION.format(self._credentials.url, sdg_id)
+
+    def get_utility_agent_tools_href(self):
+        return UTILITY_AGENT_TOOLS_BETA.format(self._get_platform_url_if_exists())
+
+    def get_utility_agent_tools_run_href(self):
+        return UTILITY_AGENT_TOOLS_RUN_BETA.format(self._get_platform_url_if_exists())
+
+    def get_jobs_runs_href(self, job_id: str, run_id: str):
+        return JOBS_RUNS.format(self._get_platform_url_if_exists(), job_id, run_id)
