@@ -280,7 +280,7 @@ class Pipelines(WMLResource):
         details = self._client.pipelines.get_details(pipeline_id)
 
         patch_payload = self.ConfigurationMetaNames._generate_patch_payload(
-            details["entity"], changes, with_validation=True
+            details, changes, with_validation=True
         )
 
         url = self._client.service_instance._href_definitions.get_pipeline_href(
