@@ -25,13 +25,13 @@ from ibm_watsonx_ai.metanames import (
     PipelineMetanames,
     SpacesMetaNames,
     ModelMetaNames,
-    MemberMetaNames,
+    RepositoryMemberMetaNames,
     AIServiceMetaNames,
 )
 from ibm_watsonx_ai.models import Models
 from ibm_watsonx_ai.pipelines import Pipelines
 from ibm_watsonx_ai.utils import inherited_docstring, get_url, get_user_agent_header
-from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid, _is_iam_token
+from ibm_watsonx_ai.utils.utils import _get_id_from_deprecated_uid
 from ibm_watsonx_ai.wml_client_error import WMLClientError
 from ibm_watsonx_ai.wml_resource import WMLResource
 
@@ -134,60 +134,20 @@ class Repository(WMLResource):
         self.PipelineMetaNames = PipelineMetanames()
         self.SpacesMetaNames = SpacesMetaNames()
         self.ModelMetaNames = ModelMetaNames()
-        self.MemberMetaNames = MemberMetaNames()
+        self.MemberMetaNames = RepositoryMemberMetaNames()
         self.AIServiceMetaNames = AIServiceMetaNames()
 
-        self._refresh_repo_client()  # regular token is initialized in service_instance
+        # make sure that old repo client is aware of token changes
+        self._client._auth_method._on_token_set = self._refresh_repo_client
+        self._client._auth_method._on_token_creation = self._refresh_repo_client
+        self._client._auth_method._on_token_refresh = self._refresh_repo_client
+
+        self._refresh_repo_client()
 
     def _refresh_repo_client(self) -> None:
-        def set_client_headers():
-            self._ml_repository_client._add_header(
-                "User-Agent", get_user_agent_header()
-            )
-
-            if self._client.default_project_id is not None:
-                self._ml_repository_client._add_header(
-                    "X-Watson-Project-ID", self._client.default_project_id
-                )
-
-        # If apiKey is passed in credentials then refresh repoclient with IAM token else MLToken
         self._ml_repository_client = MLRepositoryClient(self._credentials.url)
-        from ibm_watsonx_ai.utils.auth import TokenAuth
-
-        if isinstance(self._client.service_instance._auth_method, TokenAuth):
-            # _is_iam_token is used here in strange way
-            # it is checked if token is iam and has instance_id or is not iam token
-            if _is_iam_token(self._client.token) is not None:
-                self._ml_repository_client.authorize_with_token(self._client.token)
-            else:
-                self._ml_repository_client.authorize_with_iamtoken(
-                    self._client.token,
-                    self._credentials.instance_id,
-                    True,
-                )
-
-            set_client_headers()
-        else:
-            if self._client._is_IAM():
-                self._ml_repository_client.authorize_with_iamtoken(
-                    self._client.token,
-                    self._credentials.instance_id,
-                    True,
-                )
-                set_client_headers()
-            else:
-                if self._client.ICP_PLATFORM_SPACES:
-                    self._repotoken = self._client.token
-                    self._ml_repository_token = self._repotoken.replace("Bearer", "")
-                    self._ml_repository_client.authorize_with_token(
-                        self._ml_repository_token
-                    )
-                else:
-                    self._ml_repository_client.authorize(
-                        self._credentials.username,
-                        self._credentials.password,
-                    )
-                    set_client_headers()
+        self._ml_repository_client.authorize_with_token(self._client.token)
+        self._ml_repository_client._add_header("User-Agent", get_user_agent_header())
 
     @inherited_docstring(
         Experiments.store, {"experiments.get_href": "repository.get_experiment_href"}
@@ -810,19 +770,6 @@ class Repository(WMLResource):
             except Exception as e:
                 self._logger.error(e)
 
-        sw_spec_info = {
-            s["id"]: s
-            for s in self._client.software_specifications.get_details(state_info=True)[
-                "resources"
-            ]
-        }
-
-        def get_spec_info(spec_id: str, prop: str) -> str:
-            if spec_id and spec_id in sw_spec_info:
-                return sw_spec_info[spec_id].get(prop, "")
-            else:
-                return ""
-
         values = []
         for t in endpoints.keys():
             values += [
@@ -836,13 +783,8 @@ class Repository(WMLResource):
                         if t != "function" or t != "ai_service"
                         else m["entity"]["type"] + " function"
                     ),
-                    get_spec_info(
-                        m["entity"].get("software_spec", {}).get("id"), "state"
-                    ),
-                    get_spec_info(
-                        m["entity"].get("software_spec", {}).get("id"),
-                        "replacement",
-                    ),
+                    self._client.software_specifications._get_state(m),
+                    self._client.software_specifications._get_replacement(m),
                 )
                 for m in resources[t]
             ]
