@@ -14,7 +14,7 @@ from ibm_watsonx_ai.messages.messages import Messages
 
 from ibm_watsonx_ai.metanames import (
     SpacesMetaNames,
-    SpacesMemberMetaNames,
+    MemberMetaNames,
 )
 from ibm_watsonx_ai.utils import (
     print_text_header_h1,
@@ -40,7 +40,7 @@ class Spaces(WMLResource):
     ConfigurationMetaNames = SpacesMetaNames()
     """MetaNames for spaces creation."""
 
-    MemberMetaNames = SpacesMemberMetaNames()
+    MemberMetaNames = MemberMetaNames()
     """MetaNames for space members creation."""
 
     def __init__(self, client: APIClient):
@@ -158,10 +158,9 @@ class Spaces(WMLResource):
                 or "crn" not in meta_props["compute"]
             ):
                 raise WMLClientError("'name' and 'crn' is mandatory for 'COMPUTE'")
-            temp_meta = meta_props["compute"]
-            temp_meta.update({"type": "machine_learning"})
 
-            meta_props["compute"] = temp_meta
+            if "type" not in meta_props["compute"]:
+                meta_props["compute"]["type"] = "machine_learning"
 
         if "stage" in meta_props and self._client.CLOUD_PLATFORM_SPACES:
             if not type(meta_props["stage"]["production"]) == bool:
@@ -427,7 +426,9 @@ class Spaces(WMLResource):
         :param member: filters the result list, only includes spaces where the user with a matching user ID
             is a member
         :type member: str, optional
-        :param roles: limit number of fetched records
+        :param roles: a list of comma-separated space roles to use to filter the query results,
+            must be used in conjunction with the "member" query parameter,
+            available values : `admin`, `editor`, `viewer`
         :type roles: str, optional
         :param space_type: filter spaces by their type, available types are 'wx', 'cpd', and 'wca'
         :type space_type: str, optional
@@ -535,14 +536,10 @@ class Spaces(WMLResource):
             payload_compute.append(changes["compute"])
             changes["compute"] = payload_compute
 
-        print("changes in update: ", changes)
-
         patch_payload = self.ConfigurationMetaNames._generate_patch_payload(
             details["entity"], changes
         )
 
-        print("patch payload: ", patch_payload)
-
         href = self._client.service_instance._href_definitions.get_platform_space_href(
             space_id
         )
