@@ -84,16 +84,12 @@ class SwSpec(WMLResource):
         else:
             if state_info:
                 response = requests.get(
-                    self._client.service_instance._href_definitions.get_software_specifications_list_href(),
+                    self._client.service_instance._href_definitions.get_sw_specs_href(),
                     params=self._client._params(),
                     headers=self._client._get_headers(),
                 )
 
-                return {
-                    "resources": self._handle_response(
-                        200, "get sw specs details", response
-                    )["results"]
-                }
+                return self._handle_response(200, "get sw specs details", response)
             else:
                 response = requests.get(
                     self._client.service_instance._href_definitions.get_sw_specs_href(),
@@ -133,9 +129,9 @@ class SwSpec(WMLResource):
             meta_props = {
                 client.software_specifications.ConfigurationMetaNames.NAME: "skl_pipeline_heart_problem_prediction",
                 client.software_specifications.ConfigurationMetaNames.DESCRIPTION: "description scikit-learn_0.20",
-                client.software_specifications.ConfigurationMetaNames.PACKAGE_EXTENSIONS_UID: [],
-                client.software_specifications.ConfigurationMetaNames.SOFTWARE_CONFIGURATIONS: {},
-                client.software_specifications.ConfigurationMetaNames.BASE_SOFTWARE_SPECIFICATION_ID: "guid"
+                client.software_specifications.ConfigurationMetaNames.PACKAGE_EXTENSIONS: [],
+                client.software_specifications.ConfigurationMetaNames.SOFTWARE_CONFIGURATION: {},
+                client.software_specifications.ConfigurationMetaNames.BASE_SOFTWARE_SPECIFICATION: {"guid": "<guid>"}
             }
 
             sw_spec_details = client.software_specifications.store(meta_props)
@@ -162,12 +158,17 @@ class SwSpec(WMLResource):
 
         return sw_spec_details
 
-    def list(self, limit: int | None = None) -> DataFrame:
+    def list(
+        self, limit: int | None = None, spec_states: list[SpecStates] | None = None
+    ) -> DataFrame:
         """List software specifications in a table format.
 
         :param limit: limit number of fetched records
         :type limit: int, optional
 
+        :param spec_states: specification state filter, by default shows available, supported and custom software specifications
+        :type spec_states: list[SpecStates], optional
+
         :return: pandas.DataFrame with listed software specifications
         :rtype: pandas.DataFrame
 
@@ -177,66 +178,30 @@ class SwSpec(WMLResource):
 
             client.software_specifications.list()
         """
-        href = self._client.service_instance._href_definitions.get_sw_specs_href()
+        if spec_states is None:
+            spec_states = [SpecStates.SUPPORTED, SpecStates.AVAILABLE, ""]
 
-        response = requests.get(
-            href, params=self._client._params(), headers=self._client._get_headers()
+        spec_states = [s.value if isinstance(s, SpecStates) else s for s in spec_states]
+
+        asset_details = self.get_details(state_info=True)
+
+        sw_spec_values = [
+            (
+                m["metadata"]["name"],
+                m["metadata"]["asset_id"],
+                m["entity"]["software_specification"].get("type", "derived"),
+                self._get_spec_state(m),
+                m["metadata"].get("life_cycle", {}).get("replacement_name", ""),
+            )
+            for m in asset_details["resources"]
+            if self._get_spec_state(m) in spec_states
+        ]
+        table = self._list(
+            sw_spec_values,
+            ["NAME", "ID", "TYPE", "STATE", "REPLACEMENT"],
+            limit,
         )
 
-        asset_details = self._handle_response(200, "list assets", response)["resources"]
-
-        if self._client.CPD_version >= 4.8 or self._client.CLOUD_PLATFORM_SPACES:
-            sw_spec_values = [
-                (
-                    m["metadata"]["name"],
-                    m["metadata"]["asset_id"],
-                    m["entity"]["software_specification"].get("type", "derived"),
-                    self._get_spec_state(m),
-                    m["metadata"].get("life_cycle", {}).get("replacement_name", ""),
-                )
-                for m in asset_details
-            ]
-            table = self._list(
-                sw_spec_values,
-                ["NAME", "ID", "TYPE", "STATE", "REPLACEMENT"],
-                limit,
-            )
-
-        else:
-            href = (
-                self._client.service_instance._href_definitions.get_software_specifications_list_href()
-            )
-
-            response = requests.get(
-                href, params=self._client._params(), headers=self._client._get_headers()
-            )
-
-            spec_state_dict = {
-                el["id"]: el
-                for el in self._handle_response(
-                    200, "list sw_specs with spec_state info", response
-                )["results"]
-            }
-            sw_spec_values = [
-                (
-                    m["metadata"]["name"],
-                    m["metadata"]["asset_id"],
-                    m["entity"]["software_specification"].get("type", "derived"),
-                    spec_state_dict.get(m["metadata"]["asset_id"], {}).get(
-                        "state", "not_provided"
-                    ),
-                    spec_state_dict.get(m["metadata"]["asset_id"], {}).get(
-                        "replacement", ""
-                    ),
-                )
-                for m in asset_details
-            ]
-
-            table = self._list(
-                sw_spec_values,
-                ["NAME", "ID", "TYPE", "STATE", "REPLACEMENT"],
-                limit,
-            )
         return table
 
     @staticmethod
@@ -512,17 +477,27 @@ class SwSpec(WMLResource):
         return self._handle_response(204, "pkg spec delete", response, False)
 
     @staticmethod
-    def _get_spec_state(asset: dict) -> str:
-        if "life_cycle" in asset["metadata"]:
-            if SpecStates.RETIRED.value in asset["metadata"]["life_cycle"]:
-                return SpecStates.RETIRED.value
-            elif SpecStates.CONSTRICTED.value in asset["metadata"]["life_cycle"]:
-                return SpecStates.CONSTRICTED.value
-            elif SpecStates.DEPRECATED.value in asset["metadata"]["life_cycle"]:
-                return SpecStates.DEPRECATED.value
-            else:
-                return SpecStates.SUPPORTED.value
-        return "not_provided"
+    def _get_spec_state(spec_details: dict) -> str:
+        if spec_details["entity"].get("software_specification").get("type") != "base":
+            return ""
+        elif "life_cycle" not in spec_details["metadata"]:
+            return (
+                SpecStates.SUPPORTED.value
+            )  # if no lifecycle info in the metadata, then we should assume it is supported
+        elif SpecStates.RETIRED.value in spec_details["metadata"]["life_cycle"]:
+            return SpecStates.RETIRED.value
+        elif SpecStates.CONSTRICTED.value in spec_details["metadata"]["life_cycle"]:
+            return SpecStates.CONSTRICTED.value
+        elif SpecStates.DEPRECATED.value in spec_details["metadata"]["life_cycle"]:
+            return SpecStates.DEPRECATED.value
+        else:
+            for state in SpecStates:
+                if state.value in spec_details["metadata"]["life_cycle"]:
+                    return state.value
+
+            return (
+                SpecStates.SUPPORTED.value
+            )  # when no other info in lifecycle, then we should assume it is supported
 
     def _get_required_element_from_response(
         self, response_data: dict[str, Any]
@@ -553,3 +528,61 @@ class SwSpec(WMLResource):
                 "Failed to read Response from down-stream service: "
                 + str(response_data)
             )
+
+    def _get_state_info(self) -> tuple[dict, dict]:
+        spec_details = self.get_details(state_info=True)
+
+        state_info_by_id = {
+            s["metadata"].get("asset_id"): {
+                "state": self._get_spec_state(s),
+                "replacement": s["metadata"]
+                .get("life_cycle", {})
+                .get("replacement_name", ""),
+            }
+            for s in spec_details["resources"]
+        }
+
+        state_info_by_name = {
+            s["metadata"].get("name"): {
+                "state": self._get_spec_state(s),
+                "replacement": s["metadata"]
+                .get("life_cycle", {})
+                .get("replacement_name", ""),
+            }
+            for s in spec_details["resources"]
+        }
+
+        return state_info_by_id, state_info_by_name
+
+    def _get_info(self, asset_details) -> dict | None:
+        if not hasattr(self, "_spec_info"):
+            self._spec_info = self._get_state_info()
+
+        if (
+            sw_spec_id := asset_details.get("entity", {})
+            .get("software_spec", {})
+            .get("id")
+        ) is not None:
+            return self._spec_info[0].get(sw_spec_id)
+        elif (
+            sw_spec_name := asset_details.get("entity", {})
+            .get("software_spec", {})
+            .get("name")
+        ) is not None:
+            return self._spec_info[1].get(sw_spec_name)
+
+    def _get_state(self, asset_details) -> str:
+        spec_info = self._get_info(asset_details)
+
+        if spec_info:
+            return spec_info.get("state", "")
+        else:
+            return "not_provided"
+
+    def _get_replacement(self, asset_details) -> str:
+        spec_info = self._get_info(asset_details)
+
+        if spec_info:
+            return spec_info.get("replacement", "")
+        else:
+            return ""
