@@ -46,6 +46,15 @@ if TYPE_CHECKING:
 
 _DEFAULT_LIST_BATCH_SIZE_LIMIT = 200
 
+httpx_handled = [
+    "ibm_watsonx_ai.deployments",
+    "ibm_watsonx_ai.foundation_models.ilab.taxonomies",
+    "ibm_watsonx_ai.foundation_models.ilab.documents",
+    "ibm_watsonx_ai.foundation_models.ilab.synthetic_data",
+    "ibm_watsonx_ai.foundation_models_manager",
+    "ibm_watsonx_ai.foundation_models.inference.fm_model_inference",
+]
+
 
 class WMLResource:
     def __init__(self, name: str, client: APIClient):
@@ -384,8 +393,14 @@ class WMLResource:
 
             url = base_url + "/" + id
 
-            response_get = requests.get(
-                url, params, headers=self._client._get_headers()
+            request_handler = (
+                self._client.httpx_client if self._name in httpx_handled else requests
+            )
+
+            response_get = request_handler.get(
+                url=url,
+                params=params,
+                headers=self._client._get_headers(),
             )
 
             return self._handle_response(200, op_name, response_get)
@@ -453,6 +468,14 @@ class WMLResource:
         _filter_func: Callable | None = None,
     ) -> ArtifactDetailsType:
         params = self._client._params(skip_space_project_chk)
+
+        if self._name in httpx_handled:
+            request_handler = self._client.httpx_client
+            use_httpx = True
+        else:
+            request_handler = requests
+            use_httpx = False
+
         if query_params is not None:
             params.update(query_params)
 
@@ -489,7 +512,7 @@ class WMLResource:
         url_2 = "/".join(url.split("/")[:3])
 
         resource_generator = next_resource_generator(
-            self._client, url_2, href, params, _all, _filter_func
+            self._client, url_2, href, params, _all, _filter_func, use_httpx
         )
 
         if _async:
@@ -502,8 +525,10 @@ class WMLResource:
                 return {"resources": resources}
 
             else:
-                response_get = requests.get(
-                    url, headers=self._client._get_headers(), params=params
+                response_get = request_handler.get(
+                    url=url,
+                    headers=self._client._get_headers(),
+                    params=params,
                 )
 
                 result = cast(dict, self._handle_response(200, op_name, response_get))
@@ -730,30 +755,35 @@ class WMLResource:
         return "success"
 
     def _get_and_cache_spec_ids_for_state(self, spec_state: SpecStates) -> list:
-        if spec_state in self._client._spec_ids_per_state:
-            return self._client._spec_ids_per_state[spec_state]
+        from ibm_watsonx_ai.sw_spec import SpecStates
 
-        url = (
-            self._client.service_instance._href_definitions.get_software_specifications_list_href()
-        )
+        if isinstance(spec_state, str):
+            spec_state = SpecStates(spec_state)
 
-        params = self._client._params()
-        params["state"] = spec_state.value
+        if spec_state not in self._client._spec_ids_per_state:
+            sw_spec_details = self._client.software_specifications.get_details(
+                state_info=True
+            )
 
-        response_post = requests.get(
-            url, params=params, headers=self._client._get_headers()
-        )
+            for spec in sw_spec_details["resources"]:
+                try:
+                    state = SpecStates(
+                        self._client.software_specifications._get_spec_state(spec)
+                    )
 
-        res = cast(
-            dict,
-            self._handle_response(200, "list software specifications", response_post),
-        )
+                    if state not in self._client._spec_ids_per_state:
+                        self._client._spec_ids_per_state[state] = []
 
-        spec_ids = [r["id"] for r in res["results"]]
+                    if (
+                        sw_spec_id := spec["metadata"]["asset_id"]
+                    ) not in self._client._spec_ids_per_state[state]:
+                        self._client._spec_ids_per_state[state].append(sw_spec_id)
+                except:
+                    # The values are requested by SpecStates, so even if new state will appear
+                    # user will not request that new state. If user will wish to request new state, they will open issue.
+                    pass
 
-        self._client._spec_ids_per_state[spec_state] = spec_ids
-
-        return spec_ids
+        return self._client._spec_ids_per_state.get(spec_state, [])
 
     @staticmethod
     def _should_get_all_values(limit: int | None):
